// MgNetSelect.cpp
#include "MgNetSys.h"
#include "MgNetLib.h"

class MgNetSocket
{
public:
	int ready;
	SOCKET channel;
};

class MgNetSocketSet
{
public:
	int numsockets;
	int maxsockets;
	MgNetSocket ** sockets;
};


MgNetSocketSet * MgNet_AllocSocketSet(int maxsockets)
{
	MgNetSocketSet * set;
	int i;
	
	set = (MgNetSocketSet *)malloc(sizeof(*set));
	if (set)
	{
		set->numsockets = 0;
		set->maxsockets = maxsockets;
		set->sockets = (MgNetSocket **)malloc(maxsockets * sizeof(*set->sockets));
		if (set->sockets)
		{
			for (i = 0; i < maxsockets; i++)
				set->sockets[i] = NULL;
		}
		else
		{
			free(set);
			set = NULL;
		}
	}
	
	return set;
}

int MgNet_AddSocket(MgNetSocketSet * set, MgNetGenericSocket * sock)
{
	if (sock)
	{
		if (set->numsockets == set->maxsockets)
		{
			MgNet_SetError("Socketset is full");
			return -1;
		}
		set->sockets[set->numsockets++] = (MgNetSocket *)sock;
	}
	return set->numsockets;
}

int MgNet_DelSocket(MgNetSocketSet * set, MgNetGenericSocket * sock)
{
	int i;

	if (sock)
	{
		for (i = 0; i < set->numsockets; ++i )
		{
			if (set->sockets[i] == (MgNetSocket *)sock)
				break;
		}
		if (i == set->numsockets)
		{
			MgNet_SetError("socket not found in socketset");
			return -1;
		}
		set->numsockets--;
		for (; i < set->numsockets; ++i)
 			set->sockets[i] = set->sockets[i + 1];
	}
	return set->numsockets;
}

#ifdef MACOS_OPENTRANSPORT
int MgNet_CheckSockets(MgNetSocketSet * set, Uint32 timeout)
{
	Uint32 stop;
	int numReady;

	/* Loop, polling the network devices */
	stop = SDL_GetTicks() + timeout;
	
	do 
	{
		OTResult status;
		size_t numBytes;
		int	i;
		
		numReady = 0;
	
		for (i = set->numsockets-1; i >= 0; --i) 
		{
			status = OTCountDataBytes(set->sockets[i]->channel, &numBytes);
		
			if (status != noErr && status != kOTNoDataErr) 
			{
				return-1;
			}
			else if (numBytes > 0) 
			{
				set->sockets[i]->ready = 1;
				++numReady;
			}
		}
		
	} while (!numReady && (SDL_GetTicks() < stop));

	return numReady;
}
#else
int MgNet_CheckSockets(MgNetSocketSet * set, Uint32 timeout)
{
	int i;
	SOCKET maxfd;
	int retval;
	struct timeval tv;
	fd_set mask;

	/* Find the largest file descriptor */
	maxfd = 0;
	for (i = set->numsockets - 1; i >= 0; --i)
	{
		if (set->sockets[i]->channel > maxfd) 
			maxfd = set->sockets[i]->channel;
	}

	/* Check the file descriptors for available data */
	do
	{
		errno = 0;

		/* Set up the mask of file descriptors */
		FD_ZERO(&mask);
		for (i = set->numsockets - 1; i >= 0; --i)
			FD_SET(set->sockets[i]->channel, &mask);

		/* Set up the timeout */
		tv.tv_sec = timeout / 1000;
		tv.tv_usec = (timeout % 1000) * 1000;

		/* Look! */
		retval = select(maxfd + 1, &mask, NULL, NULL, &tv);
	} while (errno == EINTR);

	/* Mark all file descriptors ready that have data available */
	if (retval > 0) {
		for (i = set->numsockets - 1; i >= 0; --i) {
			if (FD_ISSET(set->sockets[i]->channel, &mask))
				set->sockets[i]->ready = 1;
		}
	}
	
	return retval;
}
#endif /* MACOS_OPENTRANSPORT */

void MgNet_FreeSocketSet(MgNetSocketSet * set)
{
	if (set)
	{
		free(set->sockets);
		free(set);
	}
}
