// IMG_writeJPG.cpp
#include <stdio.h>
#include "MgSurface.h"
#include "SDL_writeImage.h"


/*
 * For the C lib 'jpeglib' to link in correctly with the
 * sources, we must include it in the 'extern "C"' declaration.
 * Why???
 */

extern "C" {

#ifdef USE_JPG

#include <jpeglib.h>

int IMG_SaveJPG( FILE * fp, MgSurface * src )
{
	struct jpeg_compress_struct cinfo;
	struct jpeg_error_mgr jerr;
	
	cinfo.err = jpeg_std_error( &jerr );
	jpeg_create_compress( &cinfo );
	
	jpeg_stdio_dest( &cinfo, fp );
	cinfo.image_width = src->Width();
	cinfo.image_height = src->Height();
	cinfo.input_components = 3; 	// RGB
	cinfo.in_color_space = JCS_RGB;
	jpeg_set_defaults( &cinfo );
	jpeg_set_quality( &cinfo, 75 /* default quality */, 
						TRUE /* force baseline */ );
	jpeg_start_compress( &cinfo, TRUE );
	
	JSAMPROW row[1];
	if ( src->Format()->BytesPerPixel() == 3 )
	{
		while ( cinfo.next_scanline < cinfo.image_height )
		{
			row[0] = (JSAMPROW)(Uint8 *)src->Pixels() +
						cinfo.next_scanline * src->Pitch();
			jpeg_write_scanlines( &cinfo, row, 1 );
		}
	}
	else
	{
		Uint8 * ptr = new Uint8[cinfo.image_width * 3];
		Uint8 * vptr = NULL;
		//Uint8 r, g, b;
		MgColor col;
		while ( cinfo.next_scanline < cinfo.image_height )
		{
			vptr = ptr;
			for ( unsigned int i = 0; i < cinfo.image_width; i++ )
			{
				col = src->Format()->GetColor(src->GetPixel(i, cinfo.next_scanline));
				*vptr = col.red; vptr++;
				*vptr = col.green; vptr++;
				*vptr = col.blue; vptr++;
			}
			row[0] = (JSAMPROW)ptr;
			jpeg_write_scanlines( &cinfo, row, 1 );
		}
		delete [] ptr;
	}
	jpeg_finish_compress( &cinfo );
	jpeg_destroy_compress( &cinfo );
	return( 0 );
}

#else

int IMG_SaveJPG( FILE * fp, MgSurface * src )
{
	return( 1 );
}

#endif

};
