// IMG_writePNG.cpp
#include <stdio.h>
#include <stdlib.h>
#include "MgSurface.h"
#include "SDL_writeImage.h"

#ifdef WIN32
# include <malloc.h>
#endif

extern "C" {

#ifdef USE_PNG

#include <png.h>

int IMG_SavePNG( FILE * fp, MgSurface * src )
{
	//SDL_Surface * sur = src->fSurface;
	
	png_structp png_ptr = png_create_write_struct( PNG_LIBPNG_VER_STRING,
													NULL, NULL, NULL );
	if ( !png_ptr ) return( -1 );
	png_infop info_ptr = png_create_info_struct( png_ptr );
	if ( !info_ptr )
	{
		png_destroy_write_struct( &png_ptr, (png_infopp)NULL );
		return( -1 );
	}
	if ( setjmp( png_ptr->jmpbuf ) )
	{
		png_destroy_write_struct( &png_ptr, &info_ptr );
		return( -1 );
	}
	png_init_io( png_ptr, fp );
//	int bpp = sur->format->BitsPerPixel;
	int bpp = src->Format()->BitsPerPixel();
	if ( bpp > 16 ) bpp = 16;
//	int w = sur->w;
	int w = src->Width();
//	int h = sur->h;
	int h = src->Height();
	
	int color_type = bpp <= 8 ? PNG_COLOR_TYPE_PALETTE : PNG_COLOR_TYPE_RGB;
	bool pal = ( color_type == PNG_COLOR_TYPE_PALETTE );
	png_set_IHDR( png_ptr, info_ptr, w, h, 8, color_type, PNG_INTERLACE_NONE,
					PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT );
	
	png_colorp ppal = NULL;
	if ( pal )
	{
		//SDL_Palette * spal = sur->format->palette;
		MgPalette * spal = src->Format()->Palette();
		MgColor palColor;
		
//		ppal = new png_color[spal->ncolors];
		ppal = new png_color[spal->NumColors()];
//		for ( int i = 0; i < spal->ncolors; i++ )
		for (int i = 0; i < spal->NumColors(); i++)
		{
			palColor = spal->GetColor(i);
			
//			ppal[i].red = spal->colors[i].r;
//			ppal[i].green = spal->colors[i].g;
//			ppal[i].blue = spal->colors[i].b;
			ppal[i].red = palColor.red;
			ppal[i].green = palColor.green;
			ppal[i].blue = palColor.blue;
		}
//		png_set_PLTE( png_ptr, info_ptr, ppal, spal->ncolors );
		png_set_PLTE(png_ptr, info_ptr, ppal, spal->NumColors());
	}
	// write header
	png_write_info( png_ptr, info_ptr );
	// write the image
	png_set_packing( png_ptr );
	png_set_swap( png_ptr );
//	png_set_swap_alpha( png_ptr );
//	png_set_filler( png_ptr, 0, PNG_FILLER_BEFORE );
//	png_set_bgr( png_ptr );
	png_byte ** rows;

	rows = (png_byte **)malloc( sizeof( *rows) * h );	// just because VC++ won't 
														// have it any other way
	if ( pal )
	{
//		printf( "paletted\n" );
		for ( int row = 0; row < h; row++ )
//			rows[row] = (png_bytep)(Uint8 *)sur->pixels + row * sur->pitch;
			rows[row] = (png_bytep)src->Pixels() + row * src->Pitch();
	}
	else
	{
		for ( int row = 0; row < h; row++ )
		{
			rows[row] = new png_byte[w * 3];
			png_bytep crow = rows[row];
			for ( int col = 0; col < w; col++ )
			{
				//Uint8 r, g, b;
				MgColor color;
				color = src->Format()->GetColor(src->GetPixel(col, row));
				*crow = (png_byte)color.red; crow++;
				*crow = (png_byte)color.green; crow++;
				*crow = (png_byte)color.blue; crow++;
			}
		}
	}
//	png_write_rows( png_ptr, rows, h );
	png_write_image( png_ptr, rows );
	png_write_end( png_ptr, info_ptr );
	png_destroy_write_struct( &png_ptr, &info_ptr );
	if ( pal )
		delete [] ppal;
	else
		for ( int j = 0; j < h; j++ )
			delete [] rows[j];
	return( 0 );
}

#else

int IMG_SavePNG( FILE * fp, MgSurface * src )
{
	return( 1 );
}

#endif

};