// IMG_writePPM.cpp
#include "MgPainter.h"
#include "SDL_writeImage.h"
#include "MgColor.h"
#include <stdio.h>
#include <string.h>

//extern "C" {

#define ppmWrite fputs

#define HEADER "#Coded by Vitaliy Mikitchenko\n#Of Mage Productions\n"

static char * ppmIntToAsc( int num, char * str )
{
	int a, i, j;
	i = strlen( str );
	j = i - 1;
	while ( i > 0 )
	{
		i--;
		a = num / 10;
		str[i] = (char)( num - ( a * 10 ) + 48 );
		num = a;
	}
	while ( ( str[i] == 48 ) && ( i < j ) )
	{
		str[i] = ' ';
		i++;
	}
	return( str );
}

static void ppmColToAscii( Uint8 r, Uint8 g, Uint8 b, char * str )
{
	char ascColVal[4];
	strcpy( ascColVal, ppmIntToAsc( r, ascColVal ) );
	strcat( str, ascColVal );
	strcpy( ascColVal, ppmIntToAsc( g, ascColVal ) );
	strcat( str, ascColVal );
	strcpy( ascColVal, ppmIntToAsc( b, ascColVal ) );
	strcat( str, ascColVal );
}

static void ppmWriteBin( MgColor col, FILE * fp )
{
	fputc( col.red, fp );
	fputc( col.green, fp );
	fputc( col.blue, fp );
}

int IMG_SavePPM( const char * file, MgSurface * src, int bin )
{
	MgColor color;
	FILE * fp;
	int w = src->Width(), h = src->Height();
	fp = fopen( file, "w" );
	
	if ( !fp )
		return( -1 );
	if ( bin )
	{
		ppmWrite( "P6\n", fp );
		ppmWrite( HEADER, fp );
		fprintf( fp, "%d %d\n", w, h );
		fprintf( fp, "%d\n", 255 );
		for ( int row = 0; row < h; row++ )
		{
			for ( int col = 0; col < w; col++ )
			{
				color = src->Format()->GetColor(src->GetPixel(col, row));
				ppmWriteBin(color, fp);
			}
		}
	}
	else
	{
		ppmWrite( "P3\n", fp );
		ppmWrite( HEADER, fp );
		fprintf( fp, "%d %d\n", w, h );
		int lm;
		for ( int row = 0; row < h; row++ )
		{
			for ( int col = 0; col < w; col++ )
			{
				lm = col + 5;
				if ( lm > w )
					lm = w;
				for ( col = col; col <= lm; col++ )
				{
					color = src->Format()->GetColor(src->GetPixel(col, row));
					char asciiCol[12]; asciiCol[0] = '\0';
					ppmColToAscii(color.red, color.green, color.blue, asciiCol);
					ppmWrite(asciiCol, fp);
				}
			}
		}
	}
	fclose( fp );
	return( 0 );
}

//};