// MgAudioSystem.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGAUDIOSYSTEM_H
#define MGAUDIOSYSTEM_H

/*! \file MgAudioSystem.h
	Contains everything related to audio initilization.
*/
#include <SDL_audio.h>

#include "MgSubSystem.h"

#include "MgBeginCode.h"

//! The highest mixing volume.
#define MG_MIX_MAXVOLUME SDL_MIX_MAXVOLUME

//! The audio state enum.
enum MgAudioStatus
{
	MG_AUDIO_STOPPED = 0,	//! Stopped.
	MG_AUDIO_PLAYING,		//! Playing.
	MG_ADUIO_PAUSED			//! Paused.
};

/*! The audio system initilization class. */
class MGDLL MgAudioSystem : public MgSubSystem
{
public:
	//! Constructor. Does nothing.
	MgAudioSystem() {}
	//! Destructor. Does nothing.
	virtual ~MgAudioSystem() {}
	
	virtual bool IsRunning() const;
	virtual bool Init();
	virtual void Quit();
	
	//! Open the audio device.
	bool OpenAudio(SDL_AudioSpec & desired, SDL_AudioSpec & obtained);
	
	//! Get the status of the audio device.
	MgAudioStatus GetAudioStatus() const;
	//! Change the state of the audio.
	void PauseAudio(bool pause);
	//! Load a WAV file into the specified buffers.
	bool LoadWAV(const char * file, SDL_AudioSpec & spec, Uint8 ** audioBuf,
				 Uint32 & audioLen);
	//! Frees the 'audioBuf' loaded with LoadWAV().
	void FreeWAV(Uint8 * audioBuf);
	//! Initializes the 'cvt' with the info needed by ConvertAudio() to convert a buffer of audio data from one format to the other.
	bool BuildAudioCVT(SDL_AudioCVT & cvt, Uint16 srcFormat, Uint8 srcChannels,
						int srcRate, Uint16 dstFormat, Uint8 dstChannels, int dstRate);
	//! Convert one audio format to another.
	int ConvertAudio(SDL_AudioCVT & cvt);
	//! Mix two buffers of the playing audio format together.
	void MixAudio(Uint8 * dst, const Uint8 * src, Uint32 len, int volume);
	//! Close audio playing without shutting it down.
	void CloseAudio();
	
	//! Lock the audio device.
	void LockAudio();
	//! Unlock the audio device.
	void UnlockAudio();
	
private:
};

#include "MgEndCode.h"

#endif

