// MgWave.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "MgMixer.h"
#include "MgWave.h"
#include "MgApp.h"
#include "debug_new.h"

MgWave::MgWave(const char * file)
{
	SDL_AudioSpec spec;
	const SDL_AudioSpec * mix = MgMixer::GetSpec();
	MgAudioSystem * sys = MgApp::Audio();
	
	fSnd = NULL;	// for init check
	
	if (!mix)
		return;
		
	if (!sys->LoadWAV(file, spec, &fSnd, fSndLen))
		return;
	
	SDL_AudioCVT cvt;
	if (!sys->BuildAudioCVT(cvt, spec.format, spec.channels, spec.freq,
							mix->format, mix->channels, mix->freq) && cvt.needed)
	{
		sys->FreeWAV(fSnd);
		fSnd = NULL;
		return;
	}
	
	int ssize = ((spec.format & 0xFF) / 8) * spec.channels;
	cvt.len = fSndLen & ~(ssize - 1);
	cvt.buf = (Uint8 *)malloc(cvt.len * cvt.len_mult);
	if (!cvt.buf)
	{
		sys->FreeWAV(fSnd);
		fSnd = NULL;
		return;
	}
	
	memcpy(cvt.buf, fSnd, fSndLen);
	sys->FreeWAV(fSnd);
	fSnd = NULL;
	
	if (sys->ConvertAudio(cvt) < 0)
	{
		free(cvt.buf);
		return;
	}
	fSnd = cvt.buf;
	fSndLen = cvt.len_cvt;
	fSndPos = 0;
}


MgWave::~MgWave()
{
	MgApp::Audio()->FreeWAV(fSnd);
}

bool 
MgWave::InitCheck() const
{
	return fSnd ? true : false;
}

void 
MgWave::Mix(Uint8 * stream, int len, int volume)
{
	Uint8 * waveptr = fSnd + fSndPos;
	MgApp::Audio()->MixAudio(stream, waveptr, len, volume);
	fSndPos += len;
}

int 
MgWave::SoundPos() const
{
	return fSndPos;
}

int 
MgWave::SoundLength() const
{
	return fSndLen;
}

void 
MgWave::Rewind()
{
	fSndPos = 0;
}

Uint8 *
MgWave::Sound() const
{
	return fSnd;
}


