// MgCDROM.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/
#ifndef MGCDROM_H
#define MGCDROM_H

/*! \file MgCDROM.h
	
	Contains CD-ROM handling code.
*/

#include <SDL_cdrom.h>

#include "MgBeginCode.h"

//! Define MG_CD. It is the same as SDL_CD.
#define MG_CD SDL_CD

//! Define MgCDTrack. It is the same as SDL_CDtrack.
#define MgCDTrack SDL_CDtrack

/*! This is the CD-ROM handling class. You can use it to play audio CD's */
class MGDLL MgCDROM
{
public:
	//! Constructor.
	MgCDROM(int drive = -1);
	//! Destructor.
	~MgCDROM();
	
	//! Returns the number of drives attached to the system.
	static int NumDrives();
	//! Returns a human readable name for the drive. (I.E. /dev/disk/ide/atapi/1/master/0
	static const char * Name(int drive);
	
	//! Returns the name of the currently open drive.
	const char * Name();
	
	//! Reopen a drive.
	bool Reopen(int drive);
	
	//! Use this to find out how many tracks there are on the CD.
	int NumTracks();
	//! Get the current track.
	int CurrentTrack();
	
	//! CDROM status enum.
	enum Status
	{
		TrayEmpty = CD_TRAYEMPTY,	//!< The tray is empty.
		Stopped = CD_STOPPED,		//!< Playback is stopped.
		Playing = CD_PLAYING,		//!< Playing.
		Paused = CD_PAUSED,			//!< Playback is stopped.
		Error = CD_ERROR			//!< Error.
	};
	
	//! Get the status of the drive.
	MgCDROM::Status GetStatus();
	
	//! See if the CD is in the drive or not.
	bool CDInDrive();
	
	//! Play the given tracks.
	bool PlayTracks(int startTrack, int startFrame, int ntracks, int nframes);
	//! Play the given tracks.
	bool PlayTracks(int startTrack, int lastTrack);
	//! Play the CD starting from 'start' frame for 'length' frames.
	bool Play(int start, int length);
	//! Play just that one given track
	bool Play(int track);
	//! Pause playback.
	bool Pause();
	//! Resume playback.
	bool Resume();
	//! Stop playback.
	bool Stop();
	//! Eject the CD-ROM drive.
	bool Eject();

	//! Get the track structure for a track.
	MgCDTrack Track(int track);
	
	//! Call this before using the object to make sure it was created properly. Returns true if it was.
	bool InitCheck() const;
	
private:
	void Close();	/* closes our drive */
	bool Open(int drive);	/* opens a drive */
	
	int fDrive;		/* drive number */
	MG_CD * fCD;	/* cd struct */
	const char * fName;	/* cd drive name */
};

#include "MgEndCode.h"

#endif

