// MgEventFilter.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include "MgEventFilter.h"

#include "debug_new.h"

bool 
MgActiveEvent::Status() const
{
	return fEvent.gain == 1 ? true : false;
}

Uint8 
MgActiveEvent::State() const
{
	return fEvent.state;
}


MgActiveEvent::MgActiveEvent(SDL_ActiveEvent & e)
{
	fEvent = e;
}

//===================================================================================

Uint8 
MgKeyboardEvent::State() const
{
	return fEvent.state;
}

SDL_keysym 
MgKeyboardEvent::Key() const
{
	return fEvent.keysym;
}


MgKeyboardEvent::MgKeyboardEvent(SDL_KeyboardEvent & e)
{
	fEvent = e;
}

//===================================================================================

Uint8 
MgMouseMotionEvent::State() const
{
	return fEvent.state;
}

Uint16 
MgMouseMotionEvent::X() const
{
	return fEvent.x;
}

Uint16 
MgMouseMotionEvent::Y() const
{
	return fEvent.y;
}

Sint16 
MgMouseMotionEvent::XRel() const
{
	return fEvent.xrel;
}

Sint16 
MgMouseMotionEvent::YRel() const
{
	return fEvent.yrel;
}


MgMouseMotionEvent::MgMouseMotionEvent(SDL_MouseMotionEvent & e)
{
	fEvent = e;
}

//===================================================================================

Uint8 
MgMouseButtonEvent::Type() const
{
	return fEvent.type;
}

Uint8 
MgMouseButtonEvent::Button() const
{
	return fEvent.button;
}

Uint8 
MgMouseButtonEvent::State() const
{
	return fEvent.state;
}

Uint16 
MgMouseButtonEvent::X() const
{
	return fEvent.x;
}

Uint16 
MgMouseButtonEvent::Y() const
{
	return fEvent.y;
}


MgMouseButtonEvent::MgMouseButtonEvent(SDL_MouseButtonEvent & e)
{
	fEvent = e;
}

//===================================================================================

Uint8
MgJoyAxisEvent::Which() const
{
	return fEvent.which;
}

Uint8 
MgJoyAxisEvent::Axis() const
{
	return fEvent.axis;
}

Sint16 
MgJoyAxisEvent::Value() const
{
	return fEvent.value;
}


MgJoyAxisEvent::MgJoyAxisEvent(SDL_JoyAxisEvent & e)
{
	fEvent = e;
}

//===================================================================================

Uint8
MgJoyBallEvent::Which() const
{
	return fEvent.which;
}

Uint8 
MgJoyBallEvent::Ball() const
{
	return fEvent.ball;
}

Sint16 
MgJoyBallEvent::XRel() const
{
	return fEvent.xrel;
}

Sint16 
MgJoyBallEvent::YRel() const
{
	return fEvent.yrel;
}


MgJoyBallEvent::MgJoyBallEvent(SDL_JoyBallEvent & e)
{
	fEvent = e;
}

//===================================================================================

Uint8
MgJoyHatEvent::Which() const
{
	return fEvent.which;
}

Uint8 
MgJoyHatEvent::Hat() const
{
	return fEvent.hat;
}

Uint8 
MgJoyHatEvent::Value() const
{
	return fEvent.value;
}


MgJoyHatEvent::MgJoyHatEvent(SDL_JoyHatEvent & e)
{
	fEvent = e;
}

//===================================================================================

Uint8
MgJoyButtonEvent::Which() const
{
	return fEvent.which;
}

Uint8 
MgJoyButtonEvent::Button() const
{
	return fEvent.button;
}

Uint8 
MgJoyButtonEvent::State() const
{
	return fEvent.state;
}


MgJoyButtonEvent::MgJoyButtonEvent(SDL_JoyButtonEvent & e)
{
	fEvent = e;
}

//===================================================================================

int 
MgResizeEvent::W() const
{
	return fEvent.w;
}

int 
MgResizeEvent::H() const
{
	return fEvent.h;
}

//===================================================================================

MgResizeEvent::MgResizeEvent(SDL_ResizeEvent & e)
{
	fEvent = e;
}


MgExposeEvent::MgExposeEvent(SDL_ExposeEvent & e)
{
	fEvent = e;
}

//===================================================================================

MgQuitEvent::MgQuitEvent(SDL_QuitEvent & e)
{
	fEvent = e;
}


//===================================================================================

MgUserEvent::MgUserEvent(Uint8 _type, int _code, void * _data1, void * _data2)
{
	fEvent.type = type = _type;
	fEvent.code = code = _code;
	fEvent.data1 = data1 = _data1;
	fEvent.data2 = data2 = _data2;
}


MgUserEvent::~MgUserEvent()
{
}

bool 
MgUserEvent::Send()
{
	SDL_Event event;

	fEvent.type = type;
	fEvent.code = code;
	fEvent.data1 = data1;
	fEvent.data2 = data2;
	
	event.type = fEvent.type;
	event.user = fEvent;
	
	return SDL_PushEvent(&event) == 0 ? true : false;
}


MgUserEvent::MgUserEvent(SDL_UserEvent & e)
{
	fEvent = e;
}

// ==============================================================================

/*! Does nothing. */
MgEventFilter::MgEventFilter()
{
}

/*! Does nothing. */
MgEventFilter::~MgEventFilter()
{
}

bool 
MgEventFilter::Active(const MgActiveEvent & e)
{
	return false;	// pass it on
}

bool 
MgEventFilter::Keyboard(const MgKeyboardEvent & e)
{
	return false;	// pass it on
}

bool 
MgEventFilter::MouseMotion(const MgMouseMotionEvent & e)
{
	return false;	// pass it on
}

bool 
MgEventFilter::MouseButton(const MgMouseButtonEvent & e)
{
	return false;	// pass it on
}

bool 
MgEventFilter::JoystickAxis(const MgJoyAxisEvent & e)
{
	return false;	// pass it on
}

bool 
MgEventFilter::JoystickBall(const MgJoyBallEvent & e)
{
	return false;	// pass it on
}

bool 
MgEventFilter::JoystickHat(const MgJoyHatEvent & e)
{
	return false;	// pass it on
}

bool 
MgEventFilter::JoystickButton(const MgJoyButtonEvent & e)
{
	return false;	// pass it on
}

bool 
MgEventFilter::UserEvent(const MgUserEvent & e)
{
	return false;	// pass it on
}

bool 
MgEventFilter::ResizeEvent(const MgResizeEvent & e)
{
	return false;
}

bool 
MgEventFilter::ExposeEvent(const MgExposeEvent & e)
{
	return false;
}

bool 
MgEventFilter::QuitEvent(const MgQuitEvent & e)
{
	return false;
}

/* ========================================================================================== */


MgEvents::MgEvents()
{
}


MgEvents::~MgEvents()
{
}

void 
MgEvents::Enable(MgEventCode event)
{
	SDL_EventState(event, SDL_ENABLE);
}

void 
MgEvents::Disable(MgEventCode event)
{
	SDL_EventState(event, SDL_IGNORE);
}

bool 
MgEvents::Query(MgEventCode event)
{
	return (SDL_EventState(event, SDL_QUERY) == SDL_ENABLE) ? true : false;
}

