// MgKeyboard.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include "MgKeyboard.h"
#include "MgApp.h"

#include <SDL_keyboard.h>


MgKeyboard::MgKeyboard()
{
	fOK = false;
	if (MgApp::App())
	{
		if (MgApp::App()->AddFilter(this, MG_KEYBOARDEVENT, MG_FILTER_SYSTEM))
			fOK = true;
	}
}


MgKeyboard::~MgKeyboard()
{
	/* if created successfully and app still exists */
	if (fOK && MgApp::App())
		MgApp::App()->RemoveFilter(this, MG_FILTER_SYSTEM);
}

bool 
MgKeyboard::EnableUNICODE(bool enable)
{
	int ret = SDL_EnableUNICODE((enable) ? 1 : 0);
	switch (ret)
	{
		case 1:
			return true;
		
		case 0:
			return false;
		
		default:	/* UNICODE translation not changed */
			return enable;
	}
}

int 
MgKeyboard::EnableKeyRepeat(int delay, int interval)
{
	return SDL_EnableKeyRepeat(delay, interval);
}

Uint8 *
MgKeyboard::GetKeyState(int * numkeys)
{
	return SDL_GetKeyState(numkeys);
}

MgMod 
MgKeyboard::GetModState()
{
	return SDL_GetModState();
}

void 
MgKeyboard::SetModState(MgMod modstate)
{
	SDL_SetModState(modstate);
}

char *
MgKeyboard::GetKeyName(MgKey key)
{
	return SDL_GetKeyName(key);
}

bool 
MgKeyboard::InitCheck() const
{
	return fOK;
}

bool 
MgKeyboard::Keyboard(const MgKeyboardEvent & e)
{
	fKeyEvent = e;
	Activated((e.State() == MG_PRESSED) ? MgKeyboard::Down : MgKeyboard::Up);
	
	return true;
}

