// MgMouse.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include "MgMouse.h"
#include "MgApp.h"
#include "debug_new.h"



MgMouse::MgMouse()
{
	/* add ourselfs as a system filter */
	if (!MgApp::App())	/* no app????!!! */
	{
		fOK = false;
	}
	else
	{
		MgApp::App()->AddFilter(this, MG_MOUSEEVENT, MG_FILTER_SYSTEM);
		fOK = true;
	}
}


MgMouse::~MgMouse()
{
	/* remove ourselves if InitCheck() (fOK) is true and the app still exists */
	if (fOK && MgApp::App())
		MgApp::App()->RemoveFilter(this, MG_FILTER_SYSTEM);	
}

bool 
MgMouse::MouseMotion(const MgMouseMotionEvent & e)
{
	fMoveEvent = e;
	/* tell the user mouse was moved. */
	Activated(MgMouse::Move);
	return true;
}

bool 
MgMouse::MouseButton(const MgMouseButtonEvent & e)
{
	fButtonEvent = e;
	/* tell the user mouse was clicked */
	Activated((e.State() == MG_PRESSED) ? MgMouse::Down : MgMouse::Up);
	return true;
}

bool
MgMouse::InitCheck() const
{
	return fOK;
}

/* static mouse implementation *************************************************/

MgSMouse::MgSMouse()
{
	fStatus.fX = fStatus.fY = fStatus.fXRel = fStatus.fYRel = 0;
	fStatus.fButtons = 0;
}


MgSMouse::~MgSMouse()
{
}
MgSMouse::Status 
MgSMouse::GrabStatus()
{
	/* update our status */
	fStatus.fButtons = SDL_GetMouseState(&fStatus.fX, &fStatus.fY);
	SDL_GetRelativeMouseState(&fStatus.fXRel, &fStatus.fYRel);
	
	/* return */
	return fStatus;
}

MgSMouse::Status 
MgSMouse::GetStatus()
{
	return fStatus;
}

/*! \param x X position.
	\param y Y position.
*/
void 
MgSMouse::Warp(Uint16 x, Uint16 y)
{
	SDL_WarpMouse(x, y);
}


