// MgDraw.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGDRAW_H
#define MGDRAW_H

#include <SDL_video.h>
#include <stdio.h>

#include "MgColor.h"
#include "MgVertex.h"

#include "MgBeginCode.h"
/* The following returns can be turned into an MgColor by using MgPixelFormat's
 * functions.
 */
MGDLL Uint32 GetPixel8(SDL_Surface * sur, int x, int y);
MGDLL Uint32 GetPixel16(SDL_Surface * sur, int x, int y);
MGDLL Uint32 GetPixel24(SDL_Surface * sur, int x, int y);
MGDLL Uint32 GetPixel32(SDL_Surface * sur, int x, int y);

MGDLL void PutPixel8(SDL_Surface * sur, int x, int y, MgColor & col);
MGDLL void PutPixel16(SDL_Surface * sur, int x, int y, MgColor & col);
MGDLL void PutPixel24(SDL_Surface * sur, int x, int y, MgColor & col);
MGDLL void PutPixel32(SDL_Surface * sur, int x, int y, MgColor & col);

MGDLL void PutPixel8(SDL_Surface * sur, int x, int y, Uint32 col);
MGDLL void PutPixel16(SDL_Surface * sur, int x, int y, Uint32 col);
MGDLL void PutPixel24(SDL_Surface * sur, int x, int y, Uint32 col);
MGDLL void PutPixel32(SDL_Surface * sur, int x, int y, Uint32 col);

MGDLL void StrokeHLine8(SDL_Surface * sur, int x1, int x2, int y, MgColor & col);
MGDLL void StrokeHLine16(SDL_Surface * sur, int x1, int x2, int y, MgColor & col);
MGDLL void StrokeHLine24(SDL_Surface * sur, int x1, int x2, int y, MgColor & col);
MGDLL void StrokeHLine32(SDL_Surface * sur, int x1, int x2, int y, MgColor & col);

MGDLL void StrokeVLine8(SDL_Surface * sur, int y1, int y2, int x, MgColor & col);
MGDLL void StrokeVLine16(SDL_Surface * sur, int y1, int y2, int x, MgColor & col);
MGDLL void StrokeVLine24(SDL_Surface * sur, int y1, int y2, int x, MgColor & col);
MGDLL void StrokeVLine32(SDL_Surface * sur, int y1, int y2, int x, MgColor & col);

MGDLL void StrokeLine8(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col);
MGDLL void StrokeLine16(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col);
MGDLL void StrokeLine24(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col);
MGDLL void StrokeLine32(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col);


MGDLL void StrokeRect8(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col);
MGDLL void StrokeRect16(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col);
MGDLL void StrokeRect24(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col);
MGDLL void StrokeRect32(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col);

MGDLL void StrokeCircle8(SDL_Surface * sur, int x, int y, int radius, MgColor & col);
MGDLL void StrokeCircle16(SDL_Surface * sur, int x, int y, int radius, MgColor & col);
MGDLL void StrokeCircle24(SDL_Surface * sur, int x, int y, int radius, MgColor & col);
MGDLL void StrokeCircle32(SDL_Surface * sur, int x, int y, int radius, MgColor & col);

MGDLL void StrokeEllipse8(SDL_Surface * sur, int x, int y, int rx, int ry, MgColor & col);
MGDLL void StrokeEllipse16(SDL_Surface * sur, int x, int y, int rx, int ry, MgColor & col);
MGDLL void StrokeEllipse24(SDL_Surface * sur, int x, int y, int rx, int ry, MgColor & col);
MGDLL void StrokeEllipse32(SDL_Surface * sur, int x, int y, int rx, int ry, MgColor & col);

MGDLL void StrokeTriangle8(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & col);
MGDLL void StrokeTriangle16(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & col);
MGDLL void StrokeTriangle24(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & col);
MGDLL void StrokeTriangle32(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & col);

MGDLL void StrokePolygon8(SDL_Surface * sur, MgIVertex * vertices, int numVertices);
MGDLL void StrokePolygon16(SDL_Surface * sur, MgIVertex * vertices, int numVertices);
MGDLL void StrokePolygon24(SDL_Surface * sur, MgIVertex * vertices, int numVertices);
MGDLL void StrokePolygon32(SDL_Surface * sur, MgIVertex * vertices, int numVertices);

MGDLL void FillRect8(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col);
MGDLL void FillRect16(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col);
MGDLL void FillRect24(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col);
MGDLL void FillRect32(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col);

MGDLL void FillCircle8(SDL_Surface * sur, int x, int y, int radius, MgColor & col);
MGDLL void FillCircle16(SDL_Surface * sur, int x, int y, int radius, MgColor & col);
MGDLL void FillCircle24(SDL_Surface * sur, int x, int y, int radius, MgColor & col);
MGDLL void FillCircle32(SDL_Surface * sur, int x, int y, int radius, MgColor & col);

MGDLL void FillEllipse8(SDL_Surface * sur, int x, int y, int rx, int ry, MgColor & col);
MGDLL void FillEllipse16(SDL_Surface * sur, int x, int y, int rx, int ry, MgColor & col);
MGDLL void FillEllipse24(SDL_Surface * sur, int x, int y, int rx, int ry, MgColor & col);
MGDLL void FillEllipse32(SDL_Surface * sur, int x, int y, int rx, int ry, MgColor & col);

MGDLL void FillTriangle8(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & col);
MGDLL void FillTriangle16(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & col);
MGDLL void FillTriangle24(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & col);
MGDLL void FillTriangle32(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & col);

MGDLL void FillPolygon8(SDL_Surface * sur, MgIVertex * vertices, int n, MgColor & color);
MGDLL void FillPolygon16(SDL_Surface * sur, MgIVertex * vertices, int n, MgColor & color);
MGDLL void FillPolygon24(SDL_Surface * sur, MgIVertex * vertices, int n, MgColor & color);
MGDLL void FillPolygon32(SDL_Surface * sur, MgIVertex * vertices, int n, MgColor & color);

MGDLL void FillGradient8(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & startCol, MgColor & endCol);
MGDLL void FillGradient16(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & startCol, MgColor & endCol);
MGDLL void FillGradient24(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & startCol, MgColor & endCol);
MGDLL void FillGradient32(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & startCol, MgColor & endCol);

MGDLL void ShadeHLine8(SDL_Surface * sur, int x1, int x2, int y, MgColor & c1, MgColor & c2);
MGDLL void ShadeHLine16(SDL_Surface * sur, int x1, int x2, int y, MgColor & c1, MgColor & c2);
MGDLL void ShadeHLine24(SDL_Surface * sur, int x1, int x2, int y, MgColor & c1, MgColor & c2);
MGDLL void ShadeHLine32(SDL_Surface * sur, int x1, int x2, int y, MgColor & c1, MgColor & c2);

MGDLL void ShadeVLine8(SDL_Surface * sur, int y1, int y2, int x, MgColor & c1, MgColor & c2);
MGDLL void ShadeVLine16(SDL_Surface * sur, int y1, int y2, int x, MgColor & c1, MgColor & c2);
MGDLL void ShadeVLine24(SDL_Surface * sur, int y1, int y2, int x, MgColor & c1, MgColor & c2);
MGDLL void ShadeVLine32(SDL_Surface * sur, int y1, int y2, int x, MgColor & c1, MgColor & c2);

MGDLL void ShadeLine8(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & c1, MgColor & c2);
MGDLL void ShadeLine16(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & c1, MgColor & c2);
MGDLL void ShadeLine24(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & c1, MgColor & c2);
MGDLL void ShadeLine32(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & c1, MgColor & c2);

MGDLL void ShadeRect8(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & ul, MgColor & ur, MgColor & dl, MgColor & dr);
MGDLL void ShadeRect16(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & ul, MgColor & ur, MgColor & dl, MgColor & dr);
MGDLL void ShadeRect24(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & ul, MgColor & ur, MgColor & dl, MgColor & dr);
MGDLL void ShadeRect32(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & ul, MgColor & ur, MgColor & dl, MgColor & dr);

MGDLL void ShadeTriangle8(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & c1, MgColor & c2, MgColor & c3);
MGDLL void ShadeTriangle16(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & c1, MgColor & c2, MgColor & c3);
MGDLL void ShadeTriangle24(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & c1, MgColor & c2, MgColor & c3);
MGDLL void ShadeTriangle32(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & c1, MgColor & c2, MgColor & c3);

MGDLL void ShadeCircle8(SDL_Surface * sur, int x, int y, int r, MgColor & center, MgColor & outside);
MGDLL void ShadeCircle16(SDL_Surface * sur, int x, int y, int r, MgColor & center, MgColor & outside);
MGDLL void ShadeCircle24(SDL_Surface * sur, int x, int y, int r, MgColor & center, MgColor & outside);
MGDLL void ShadeCircle32(SDL_Surface * sur, int x, int y, int r, MgColor & center, MgColor & outside);

MGDLL void ShadePolygon8(SDL_Surface * sur, MgIVertex * vertices, int n);
MGDLL void ShadePolygon16(SDL_Surface * sur, MgIVertex * vertices, int n);
MGDLL void ShadePolygon24(SDL_Surface * sur, MgIVertex * vertices, int n);
MGDLL void ShadePolygon32(SDL_Surface * sur, MgIVertex * vertices, int n);

MGDLL void ShadeCircleOutline8(SDL_Surface * sur, int x, int y, int r, MgColor & ctop, MgColor & cright, MgColor & cbottom, MgColor & cleft);
MGDLL void ShadeCircleOutline16(SDL_Surface * sur, int x, int y, int r, MgColor & ctop, MgColor & cright, MgColor & cbottom, MgColor & cleft);
MGDLL void ShadeCircleOutline24(SDL_Surface * sur, int x, int y, int r, MgColor & ctop, MgColor & cright, MgColor & cbottom, MgColor & cleft);
MGDLL void ShadeCircleOutline32(SDL_Surface * sur, int x, int y, int r, MgColor & ctop, MgColor & cright, MgColor & cbottom, MgColor & cleft);

MGDLL void ShadeRectOutline8(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & topleft, MgColor & topright, MgColor & bottomright, MgColor & bottomleft);
MGDLL void ShadeRectOutline16(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & topleft, MgColor & topright, MgColor & bottomright, MgColor & bottomleft);
MGDLL void ShadeRectOutline24(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & topleft, MgColor & topright, MgColor & bottomright, MgColor & bottomleft);
MGDLL void ShadeRectOutline32(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & topleft, MgColor & topright, MgColor & bottomright, MgColor & bottomleft);

#include "MgEndCode.h"

#endif

