// MgDraw_fill.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

/* this file defines all the functions that draw a filled shape */
#ifdef DO_FUNCS


#if defined(MGDRAW_8_BIT)

#define FillRect FillRect8
#define FillCircle FillCircle8
#define FillEllipse FillEllipse8
#define FillTriangle FillTriangle8
#define FillGradient FillGradient8
#define FillPolygon FillPolygon8

#elif defined(MGDRAW_16_BIT)

#define FillRect FillRect16
#define FillCircle FillCircle16
#define FillEllipse FillEllipse16
#define FillTriangle FillTriangle16
#define FillGradient FillGradient16
#define FillPolygon FillPolygon16

#elif defined(MGDRAW_24_BIT)

#define FillRect FillRect24
#define FillCircle FillCircle24
#define FillEllipse FillEllipse24
#define FillTriangle FillTriangle24
#define FillGradient FillGradient24
#define FillPolygon FillPolygon24

#elif defined(MGDRAW_32_BIT)

#define FillRect FillRect32
#define FillCircle FillCircle32
#define FillEllipse FillEllipse32
#define FillTriangle FillTriangle32
#define FillGradient FillGradient32
#define FillPolygon FillPolygon32

#endif

void
FillRect(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col)
{
	if (col.alpha == 255)
	{
		Uint32 pixel;
		MAKE_COL(sur, pixel, col.red, col.green, col.blue);

		for (int y = y1; y <= y2; y++)
		{
			for (int x = x1; x <= x2; x++)
			{
				PutPixel(sur, x, y, pixel);
			}
		}
	}
	else
	{
		for (int y = y1; y <= y2; y++)
		{
			for (int x = x1; x <= x2; x++)
			{
				PutPixel(sur, x, y, col);
			}
		}
	}
}

void
FillCircle(SDL_Surface * sur, int x, int y, int radius, MgColor & col)
{
	int cx = 0, cy = radius, df = 1 - radius, de = 3, dse = -2 * radius + 5;
	int i;	// "x"
	int rightx, leftx, cury;
	
	if (col.alpha == 255)
	{
		Uint32 pixel;
		MAKE_COL(sur, pixel, col.red, col.green, col.blue);

		do
		{
			leftx = x - cx;
			rightx = x + cx;
			cury = y + cy;
			for (i = leftx; i <= rightx; i++)
				PutPixel(sur, i, cury, pixel);
			
			leftx = x - cx;
			rightx = x + cx;
			cury = y - cy;
			for (i = leftx; i <= rightx; i++)
				PutPixel(sur, i, cury, pixel);

			leftx = x - cy;
			rightx = x + cy;
			cury = y - cx;
			for (i = leftx; i <= rightx; i++)
				PutPixel(sur, i, cury, pixel);

			leftx = x - cy;
			rightx = x + cy;
			cury = y + cx;
			for (i = leftx; i <= rightx; i++)
				PutPixel(sur, i, cury, pixel);

			if (df < 0)
			{
				df += de;
				de += 2;
				dse += 2;
			}
			else
			{
				df += dse;
				de += 2;
				dse += 4;
				cy--;
			}
			cx++;
		} while (cx <= cy);
	}
	else
	{
		do
		{
			leftx = x - cx;
			rightx = x + cx;
			cury = y + cy;
			for (i = leftx; i <= rightx; i++)
				PutPixel(sur, i, cury, col);
			
			leftx = x - cx;
			rightx = x + cx;
			cury = y - cy;
			for (i = leftx; i <= rightx; i++)
				PutPixel(sur, i, cury, col);

			leftx = x - cy;
			rightx = x + cy;
			cury = y - cx;
			for (i = leftx; i <= rightx; i++)
				PutPixel(sur, i, cury, col);

			leftx = x - cy;
			rightx = x + cy;
			cury = y + cx;
			for (i = leftx; i <= rightx; i++)
				PutPixel(sur, i, cury, col);

			if (df < 0)
			{
				df += de;
				de += 2;
				dse += 2;
			}
			else
			{
				df += dse;
				de += 2;
				dse += 4;
				cy--;
			}
			cx++;
		} while (cx <= cy);
	}
}

void
FillEllipse(SDL_Surface * sur, int x, int y, int rx, int ry, MgColor & col)
{
	int ix, iy, h, i, j, k, oh, oi, oj, ok;
	
	if (rx < 1) rx = 1;
	if (ry < 1) ry = 1;
	h = i = j = k = 0xFFFF;

	int thex;	// "x"
	int rightx, leftx, cury;
	
	if (col.alpha == 255)
	{
		Uint32 pixel;
		MAKE_COL(sur, pixel, col.red, col.green, col.blue);

		if (rx > ry)
		{
			ix = 0;
			iy = rx * 64;
			do
			{
				oh = h; oi = i; oj = j; ok = k;
				h = (ix + 32) >> 6;
				i = (iy + 32) >> 6;
				j = (h * ry) / rx;
				k = (i * ry) / rx;
				if (((h != 0) || (k != ok)) && (h < oi))
				{
					leftx = x - h;
					rightx = x + h;
					cury = y + k;
					for (thex = leftx; thex <= rightx; thex++)
						PutPixel(sur, thex, cury, pixel);
						
					if (k)
					{
						leftx = x - h;
						rightx = x + h;
						cury = y - k;
						for (thex = leftx; thex <= rightx; thex++)
							PutPixel(sur, thex, cury, pixel);
					}
				}
				if (((i != oi) || (j != oj)) && (h < i))
				{
					leftx = x - i;
					rightx = x + i;
					cury = y + j;
					for (thex = leftx; thex <= rightx; thex++)
						PutPixel(sur, thex, cury, pixel);

					if (j)
					{
						leftx = x - i;
						rightx = x + i;
						cury = y - j;
						for (thex = leftx; thex <= rightx; thex++)
							PutPixel(sur, thex, cury, pixel);
					}
				}
				ix = ix + iy / rx;
				iy = iy - ix / rx;
			} while (i > h);
		}
		else
		{
			ix = 0;
			iy = ry * 64;
			do
			{
				oh = h; oi = i; oj = j; ok = k;
				h = (ix + 32) >> 6;
				i = (iy + 32) >> 6;
				j = (h * rx) / ry;
				k = (i * rx) / ry;
				if (((j != oj) || (i != oi)) && (h < i))
				{
					leftx = x - j;
					rightx = x + j;
					cury = y + i;
					for (thex = leftx; thex <= rightx; thex++)
						PutPixel(sur, thex, cury, pixel);

					if (i)
					{
						leftx = x - j;
						rightx = x + j;
						cury = y - i;
						for (thex = leftx; thex <= rightx; thex++)
							PutPixel(sur, thex, cury, pixel);
					}
				}
				if (((k != ok) || (h != oh)) && (h < oi))
				{
					leftx = x - k;
					rightx = x + k;
					cury = y + h;
					for (thex = leftx; thex <= rightx; thex++)
						PutPixel(sur, thex, cury, pixel);

					if (h)
					{
						leftx = x - k;
						rightx = x + k;
						cury = y - h;
						for (thex = leftx; thex <= rightx; thex++)
							PutPixel(sur, thex, cury, pixel);
					}
				}
				ix = ix + iy / ry;
				iy = iy - ix / ry;
			} while (i > h);
		}
	}
	else
	{
		if (rx > ry)
		{
			ix = 0;
			iy = rx * 64;
			do
			{
				oh = h; oi = i; oj = j; ok = k;
				h = (ix + 32) >> 6;
				i = (iy + 32) >> 6;
				j = (h * ry) / rx;
				k = (i * ry) / rx;
				if (((h != 0) || (k != ok)) && (h < oi))
				{
					leftx = x - h;
					rightx = x + h;
					cury = y + k;
					for (thex = leftx; thex <= rightx; thex++)
						PutPixel(sur, thex, cury, col);
						
					if (k)
					{
						leftx = x - h;
						rightx = x + h;
						cury = y - k;
						for (thex = leftx; thex <= rightx; thex++)
							PutPixel(sur, thex, cury, col);
					}
				}
				if (((i != oi) || (j != oj)) && (h < i))
				{
					leftx = x - i;
					rightx = x + i;
					cury = y + j;
					for (thex = leftx; thex <= rightx; thex++)
						PutPixel(sur, thex, cury, col);

					if (j)
					{
						leftx = x - i;
						rightx = x + i;
						cury = y - j;
						for (thex = leftx; thex <= rightx; thex++)
							PutPixel(sur, thex, cury, col);
					}
				}
				ix = ix + iy / rx;
				iy = iy - ix / rx;
			} while (i > h);
		}
		else
		{
			ix = 0;
			iy = ry * 64;
			do
			{
				oh = h; oi = i; oj = j; ok = k;
				h = (ix + 32) >> 6;
				i = (iy + 32) >> 6;
				j = (h * rx) / ry;
				k = (i * rx) / ry;
				if (((j != oj) || (i != oi)) && (h < i))
				{
					leftx = x - j;
					rightx = x + j;
					cury = y + i;
					for (thex = leftx; thex <= rightx; thex++)
						PutPixel(sur, thex, cury, col);

					if (i)
					{
						leftx = x - j;
						rightx = x + j;
						cury = y - i;
						for (thex = leftx; thex <= rightx; thex++)
							PutPixel(sur, thex, cury, col);
					}
				}
				if (((k != ok) || (h != oh)) && (h < oi))
				{
					leftx = x - k;
					rightx = x + k;
					cury = y + h;
					for (thex = leftx; thex <= rightx; thex++)
						PutPixel(sur, thex, cury, col);

					if (h)
					{
						leftx = x - k;
						rightx = x + k;
						cury = y - h;
						for (thex = leftx; thex <= rightx; thex++)
							PutPixel(sur, thex, cury, col);
					}
				}
				ix = ix + iy / ry;
				iy = iy - ix / ry;
			} while (i > h);
		}
	}
}

void
FillTriangle(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & col)
{
	// from PowerPak
	// modified to work with libMgTk
	float y, topx, topy, midx, midy, botx, boty, ebleft, ebright, rpos, lpos;
	
	// shut the compiler up...
	y = topx = topy = midx = midy = botx = boty = ebleft = ebright = rpos = lpos = 0.0;
	
	if (y2 <= y1)
	{
		topx = x2;
		topy = y2;
		botx = x1;
		boty = y1;
	}
	else if (y2 > y1)
	{
		topx = x1;
		topy = y1;
		botx = x2;
		boty = y2;
	}
	
	if (y3 >= topy && y3 <= boty)
	{
		midx = x3;
		midy = y3;
	}
	else if (y3 < topy)
	{
		midx = topx;
		midy = topy;
		topx = x3;
		topy = y3;
	}
	else if (y3 > boty)
	{
		midx = botx;
		midy = boty;
		botx = x3;
		boty = y3;
	}
	
	ebleft = (midx - topx) / (midy - topy);
	ebright = (botx - topx) / (boty - topy);
	
	printf("ebleft = %f, ebright = %f\n", ebleft, ebright);
	rpos = topx;
	lpos = topx;
	
	Uint32 pixel;	// define even if alpha
	int thex;
	
	pixel = 0;	// to avoid warnings
	
	if (col.alpha == 255)
	{
		MAKE_COL(sur, pixel, col.red, col.green, col.blue);

		for (y = topy; y <= midy; y++)
		{
			for (thex = (int)lpos; thex <= (int)rpos; thex++)
				PutPixel(sur, thex, (int)y, pixel);
			rpos += ebright;
			lpos += ebleft;
		}
	}
	else
	{
		for (y = topy; y <= midy; y++)
		{
			for (thex = (int)lpos; thex <= (int)rpos; thex++)
				PutPixel(sur, thex, (int)y, col);
			rpos += ebright;
			lpos += ebleft;
		}
	}
	
	ebleft = (botx - midx) / (boty - midy);
	lpos = midx;
	
	if (col.alpha == 255)
	{
		for (y = midy; y <= boty; y++)
		{
			for (thex = (int)lpos; thex <= (int)rpos; thex++)
				PutPixel(sur, thex, (int)y, pixel);
			rpos += ebright;
			lpos += ebleft;
		}
	}
	else
	{
		for (y = midy; y <= boty; y++)
		{
			for (thex = (int)lpos; thex <= (int)rpos; thex++)
				PutPixel(sur, thex, (int)y, col);
			rpos += ebright;
			lpos += ebleft;
		}
	}
}

void
FillGradient(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & startCol, MgColor & endCol)
{
	int startX = x1, endX = x2;
	int startY = y1, endY = y2;
	Uint8 sr, sg, sb, sa;
	Uint8 er, eg, eb, ea;
	float cr, cg, cb, ca;
	float curr, curg, curb, cura;
	float yDif;
	int curY;
	
	sr = startCol.red;
	sg = startCol.green;
	sb = startCol.blue;
	sa = startCol.alpha;
	
	er = endCol.red;
	eg = endCol.green;
	eb = endCol.blue;
	ea = endCol.alpha;
	
	cr = cg = cb = ca = 0.0;
	if (startY >= endY || startX >= endX) return;
	yDif = (float)(endY - startY);
	if (sr > er)
	{
		cr = (float)(sr - er) / yDif;
		cr = -cr;
	}
	else if (er > sr)
	{
		cr = (float)(er - sr) / yDif;
	}
	if (sg > eg)
	{
		cg = (float)(sg - eg) / yDif;
		cg = -cg;
	}
	else if (eg > sg)
	{
		cg = (float)(eg - sg) / yDif;
	}
	if (sb > eb)
	{
		cb = (float)(sb - eb) / yDif;
		cb = -cb;
	}
	else if (eb > sb)
	{
		cb = (float)(eb - sb) / yDif;
	}
	if (sa > ea)
	{
		ca = (float)(sa - ea) / yDif;
		ca = -ca;
	}
	else if (ea > sa)
	{
		ca = (float)(ea - sa) / yDif;
	}

	curr = sr; curg = sg; curb = sb; cura = sa;
	curY = startY;
	
	MgColor curColor;
	
	Uint32 pixel;
	for (float i = 0.0; i <= yDif; i++)
	{
		curColor((Uint8)curr, (Uint8)curg, (Uint8)curb, (Uint8)cura);
		
		{	// strokehline, manuall inlined
			int x1 = startX, x2 = endX;
			if (x1 > x2)
				SWAP(x1, x2, int)
			if (curColor.alpha == 255)
			{
				MAKE_COL(sur, pixel, curColor.red, curColor.green, curColor.blue);
				for (int px = x1; px <= x2; px++)
					PutPixel(sur, px, curY, pixel);
			}
			else
			{
				for (int px = x1; px <= x2; px++)
					PutPixel(sur, px, curY, curColor);
			}
		}

		//StrokeHLine(sur, startX, endX, curY, curColor);
//		_DrawHLine(startX, endX, curY, MapRGB((Uint8)curr, (Uint8)curg, (Uint8)curb));
		curY++;
		curr += cr;
		curg += cg;
		curb += cb;
		cura += ca;
	}
}

/* function for sorting ints (exported as a C function) */

extern "C" int PolyCompareInt(const void * a, const void * b);

void
FillPolygon(SDL_Surface * sur, MgIVertex * vertices, int n, MgColor & color)
{
	int * polyInts = NULL;
	
	int i, y, miny, maxy, x1, y1, x2, y2, ind1, ind2, ints;
	
	/* make sure we're a poly */
	if (n < 3)
		return;
	
	/* allocate our array */
	polyInts = new int[n];
	
	/* get y maximum */
	miny = vertices[0].y;
	maxy = vertices[0].y;
	
	for (i = 0; i < n; i++)
	{
		if (vertices[i].y < miny)
		{
			miny = vertices[i].y;
		}
		else if (vertices[i].y > maxy)
		{
			maxy = vertices[i].y;
		}
	}
	
	/* draw the polygon */
	for (y = miny; y <= maxy; y++)
	{
		/* do a scan first */
		ints = 0;
		for (i = 0; i < n; i++)
		{
			if (!i)
			{
				ind1 = n - 1;
				ind2 = 0;
			}
			else
			{
				ind1 = i - 1;
				ind2 = i;
			}
			y1 = vertices[ind1].y;
			y2 = vertices[ind2].y;
			if (y1 < y2)
			{
				x1 = vertices[ind1].x;
				x2 = vertices[ind2].x;
			}
			else if (y1 > y2)
			{
				y2 = vertices[ind1].y;
				y1 = vertices[ind2].y;
				x2 = vertices[ind1].x;
				x1 = vertices[ind2].x;
			}
			else
			{
				continue;
			}
			
			if ((y >= y1 && y < y2) || (y == maxy && y > y1 && y <= y2))
				polyInts[ints++] = (y - y1) * (x2 - x1) / (y2 - y1) + x1;
		}
		/* run a quick sort of the points */
		qsort(polyInts, ints, sizeof(int), PolyCompareInt);
		
		/* now draw */
		for (i = 0; i < ints; i += 2)
			STROKE_H_LINE(sur, polyInts[i], polyInts[i + 1], y, color);
	}
	
	delete [] polyInts;
}

#if defined(MGDRAW_8_BIT)	/* do this so we don't redefine it 4 times (one for each bit-depth) */
int
PolyCompareInt(const void * a, const void * b)
{	
	return (*(const int *)a) - (*(const int *)b);
}
#endif

#undef FillRect
#undef FillCircle
#undef FillTriangle
#undef FillEllipse
#undef FillGradient
#undef FillPolygon

#endif