// MgDraw_pixel_inline.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/


/* This file contains all the inline or macro pixel handling code. */
#define EXTRACT_RGB(sur, col, r, g, b) \
		r = (col & sur->format->Rmask) >> sur->format->Rshift << sur->format->Rloss; \
		g = (col & sur->format->Gmask) >> sur->format->Gshift << sur->format->Gloss; \
		b = (col & sur->format->Bmask) >> sur->format->Bshift << sur->format->Bloss

#define MAKE_COL(sur, col, r, g, b) \
		col = (((r >> sur->format->Rloss) << sur->format->Rshift) | \
				((g >> sur->format->Gloss) << sur->format->Gshift) | \
				((b >> sur->format->Bloss) << sur->format->Bshift) | \
				((255 >> sur->format->Aloss) << sur->format->Ashift))	// added the alpha so that surfaces with alpha channels will work correctly

#define SWAP(x, y, type) \
	{ \
		type swap = x; \
		x = y; \
		y = swap; \
	}

#if defined(MGDRAW_8_BIT) || defined(MGDRAW_16_BIT) || defined(MGDRAW_24_BIT) || defined(MGDRAW_32_BIT)
// clipping routines
inline bool clip_x(SDL_Surface * sur, int x)
{
	if (x > (sur->clip_rect.x + sur->clip_rect.w - 1) || x < sur->clip_rect.x)
		return true;	// clipped
	return false;
}
inline bool clip_y(SDL_Surface * sur, int y)
{
	if (y > (sur->clip_rect.y + sur->clip_rect.h - 1) || y < sur->clip_rect.y)
		return true;	// clipped
	return false;
}

#define clip_y_min(sur) sur->clip_rect.y
#define clip_y_max(sur) (sur->clip_rect.y + sur->clip_rect.h - 1)
#define clip_x_min(sur) sur->clip_rect.x
#define clip_x_max(sur) (sur->clip_rect.x + sur->clip_rect.w - 1)

#endif


// pixel routines
#if defined(MGDRAW_8_BIT)

inline Uint32
_GetPixel8(SDL_Surface * sur, int x, int y)
{
	if (clip_x(sur, x) || clip_y(sur, y))
		return 0;
	return *((Uint8 *)sur->pixels + y * sur->pitch + x);
}

// internal putpixel
inline void
_PutPixel8(SDL_Surface * sur, int x, int y, MgColor & col)
{
	if (clip_x(sur, x) || clip_y(sur, y))
		return;

	Uint32 ucol;
	ucol = SDL_MapRGB(sur->format, col.red, col.green, col.blue);
	
	*((Uint8 *)sur->pixels + y * sur->pitch + x) = ucol;
}

inline void
_PutPixel8(SDL_Surface * sur, int x, int y, Uint32 col)
{
	if (clip_x(sur, x) || clip_y(sur, y))
		return;

	*((Uint8 *)sur->pixels + y * sur->pitch + x) = col;
}

#elif defined(MGDRAW_16_BIT)

inline Uint32
_GetPixel16(SDL_Surface * sur, int x, int y)
{
	if (clip_x(sur, x) || clip_y(sur, y))
		return 0;

	return *(Uint16 *)((Uint8 *)sur->pixels + y * sur->pitch + x * 2);
}

// alpha version
inline void
_PutPixel16(SDL_Surface * sur, int x, int y, MgColor & col)
{
	if (clip_x(sur, x) || clip_y(sur, y))
		return;

	Uint32 srcCol = _GetPixel16(sur, x, y);
	Uint8 r, g, b;
	EXTRACT_RGB(sur, srcCol, r, g, b);
		
	float srcBlend, dstBlend;	// dstBlend == passed col
	dstBlend = (float)col.alpha / 255.0;
	srcBlend = 1.0 - dstBlend;
		
	r = (Uint8)(r * srcBlend + col.red * dstBlend);
	g = (Uint8)(g * srcBlend + col.green * dstBlend);
	b = (Uint8)(b * srcBlend + col.blue * dstBlend);
	MAKE_COL(sur, srcCol, r, g, b);
	*(Uint16 *)((Uint8 *)sur->pixels + y * sur->pitch + x * 2) = srcCol;
}

// non-alpha version
inline void
_PutPixel16(SDL_Surface * sur, int x, int y, Uint32 col)
{
	if (clip_x(sur,x ) || clip_y(sur, y))
		return;

	*(Uint16 *)((Uint8 *)sur->pixels + y * sur->pitch + x * 2) = col;
}
	
#elif defined(MGDRAW_24_BIT)

inline Uint32
_GetPixel24(SDL_Surface * sur, int x, int y)
{
	if (clip_x(sur, x) || clip_y(sur, y))
		return 0;

	Uint8 * p;
	int shift;
	Uint32 c = 0;
	
	p = (Uint8 *)sur->pixels + y * sur->pitch + x * 3;
	shift = sur->format->Rshift;
	c = *(p + shift / 8) << shift;
	shift = sur->format->Gshift;
	c |= *(p + shift / 8) << shift;
	shift = sur->format->Bshift;
	c |= *(p + shift / 8) << shift;
	return c;
}

inline void
_PutPixel24(SDL_Surface * sur, int x, int y, MgColor & col)
{
	if (clip_x(sur, x) || clip_y(sur, y))
		return;

	Uint8 * p;
	int shift;
	
	Uint32 srcCol = _GetPixel24(sur, x, y);
	Uint8 r, g, b;
	EXTRACT_RGB(sur, srcCol, r, g, b);
	
	float srcBlend, dstBlend;	// dstBlend == passed col
	dstBlend = (float)col.alpha / 255.0;
	srcBlend = 1.0 - dstBlend;
		
	r = (Uint8)(r * srcBlend + col.red * dstBlend);
	g = (Uint8)(g * srcBlend + col.green * dstBlend);
	b = (Uint8)(b * srcBlend + col.blue * dstBlend);
	MAKE_COL(sur, srcCol, r, g, b);		

	p = (Uint8 *)sur->pixels + y * sur->pitch + x * 3;
	shift = sur->format->Rshift;
	*(p + shift / 8) = srcCol >> shift;
	shift = sur->format->Gshift;
	*(p + shift / 8) = srcCol >> shift;
	shift = sur->format->Bshift;
	*(p + shift / 8) = srcCol >> shift;
}

// no alpha
inline void
_PutPixel24(SDL_Surface * sur, int x, int y, Uint32 col)
{
	if (clip_x(sur, x) || clip_y(sur, y))
		return;

	Uint8 * p;
	int shift;

	p = (Uint8 *)sur->pixels + y * sur->pitch + x * 3;
	shift = sur->format->Rshift;
	*(p + shift / 8) = col >> shift;
	shift = sur->format->Gshift;
	*(p + shift / 8) = col >> shift;
	shift = sur->format->Bshift;
	*(p + shift / 8) = col >> shift;
}

#elif defined(MGDRAW_32_BIT)

inline Uint32
_GetPixel32(SDL_Surface * sur, int x, int y)
{
	if (clip_x(sur, x) || clip_y(sur, y))
		return 0;

	return *(Uint32 *)((Uint8 *)sur->pixels + y * sur->pitch + x * 4);
}

// alpha
inline void
_PutPixel32(SDL_Surface * sur, int x, int y, MgColor & col)
{
	if (clip_x(sur, x) || clip_y(sur, y))
		return;

	Uint32 srcCol = _GetPixel32(sur, x, y);
	Uint8 r, g, b;
	EXTRACT_RGB(sur, srcCol, r, g, b);
		
	float srcBlend, dstBlend;	// dstBlend == passed col
	dstBlend = (float)col.alpha / 255.0;
	srcBlend = 1.0 - dstBlend;
		
	r = (Uint8)(r * srcBlend + col.red * dstBlend);
	g = (Uint8)(g * srcBlend + col.green * dstBlend);
	b = (Uint8)(b * srcBlend + col.blue * dstBlend);
		
	MAKE_COL(sur, srcCol, r, g, b);
	*(Uint32 *)((Uint8 *)sur->pixels + y * sur->pitch + x * 4) = srcCol;
}
	
// no alpha
inline void
_PutPixel32(SDL_Surface * sur, int x, int y, Uint32 col)
{
	if (clip_x(sur, x) || clip_y(sur, y))
		return;

	*(Uint32 *)((Uint8 *)sur->pixels + y * sur->pitch + x * 4) = col;
}

#endif


#ifdef DO_FUNCS

#if defined(MGDRAW_8_BIT)
# define PutPixel _PutPixel8
# define GetPixel _GetPixel8
#elif defined(MGDRAW_16_BIT)
# define PutPixel _PutPixel16
# define GetPixel _GetPixel16
#elif defined(MGDRAW_24_BIT)
# define PutPixel _PutPixel24
# define GetPixel _GetPixel24
#elif defined(MGDRAW_32_BIT)
# define PutPixel _PutPixel32
# define GetPixel _GetPixel32
#endif

extern void StrokeLine8(SDL_Surface *, int, int, int, int, MgColor &);
extern void StrokeLine16(SDL_Surface *, int, int, int, int, MgColor &);
extern void StrokeLine24(SDL_Surface *, int, int, int, int, MgColor &);
extern void StrokeLine32(SDL_Surface *, int, int, int, int, MgColor &);

extern void StrokeHLine8(SDL_Surface *, int, int, int, MgColor &);
extern void StrokeHLine16(SDL_Surface *, int, int, int, MgColor &);
extern void StrokeHLine24(SDL_Surface *, int, int, int, MgColor &);
extern void StrokeHLine32(SDL_Surface *, int, int, int, MgColor &);

extern void ShadeHLine8(SDL_Surface *, int, int, int, MgColor &, MgColor &);
extern void ShadeHLine16(SDL_Surface *, int, int, int, MgColor &, MgColor &);
extern void ShadeHLine24(SDL_Surface *, int, int, int, MgColor &, MgColor &);
extern void ShadeHLine32(SDL_Surface *, int, int, int, MgColor &, MgColor &);

extern void ShadeVLine8(SDL_Surface *, int, int, int, MgColor &, MgColor &);
extern void ShadeVLine16(SDL_Surface *, int, int, int, MgColor &, MgColor &);
extern void ShadeVLine24(SDL_Surface *, int, int, int, MgColor &, MgColor &);
extern void ShadeVLine32(SDL_Surface *, int, int, int, MgColor &, MgColor &);

#if defined(MGDRAW_8_BIT)

#define STROKE_LINE StrokeLine8
#define STROKE_V_LINE StrokeVLine8
#define STROKE_H_LINE StrokeHLine8
#define SHADE_H_LINE ShadeHLine8
#define SHADE_V_LINE ShadeVLine8

#elif defined(MGDRAW_16_BIT)

#define STROKE_LINE StrokeLine16
#define STROKE_V_LINE StrokeVLine16
#define STROKE_H_LINE StrokeHLine16
#define SHADE_H_LINE ShadeHLine16
#define SHADE_V_LINE ShadeVLine16

#elif defined(MGDRAW_24_BIT)

#define STROKE_LINE StrokeLine24
#define STROKE_V_LINE StrokeVLine24
#define STROKE_H_LINE StrokeHLine24
#define SHADE_H_LINE ShadeHLine24
#define SHADE_V_LINE ShadeVLine24

#elif defined(MGDRAW_32_BIT)

#define STROKE_LINE StrokeLine32
#define STROKE_V_LINE StrokeVLine32
#define STROKE_H_LINE StrokeHLine32
#define SHADE_H_LINE ShadeHLine32
#define SHADE_V_LINE ShadeVLine32

#endif

template <class type>
inline type
abs(type v)
{
	return (v < 0) ? -v : v;
}

/* end inlines */

#endif
