// MgDraw_shade.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGDRAW_SHADE_H
#define MGDRAW_SHADE_H

/* this file defines all the functions that do gouraud shading */
#ifdef DO_FUNCS


#if defined(MGDRAW_8_BIT)

#define ShadeHLine ShadeHLine8
#define ShadeVLine ShadeVLine8
#define ShadeLine ShadeLine8
#define ShadeRect ShadeRect8
#define ShadeTriangle ShadeTriangle8
#define ShadeCircle ShadeCircle8
#define ShadePolygon ShadePolygon8
#define ShadeCircleOutline ShadeCircleOutline8
#define ShadeRectOutline ShadeRectOutline8
#elif defined(MGDRAW_16_BIT)

#define ShadeHLine ShadeHLine16
#define ShadeVLine ShadeVLine16
#define ShadeLine ShadeLine16
#define ShadeRect ShadeRect16
#define ShadeTriangle ShadeTriangle16
#define ShadeCircle ShadeCircle16
#define ShadePolygon ShadePolygon16
#define ShadeCircleOutline ShadeCircleOutline16
#define ShadeRectOutline ShadeRectOutline16

#elif defined(MGDRAW_24_BIT)

#define ShadeHLine ShadeHLine24
#define ShadeVLine ShadeVLine24
#define ShadeLine ShadeLine24
#define ShadeRect ShadeRect24
#define ShadeTriangle ShadeTriangle24
#define ShadeCircle ShadeCircle24
#define ShadePolygon ShadePolygon24
#define ShadeCircleOutline ShadeCircleOutline24
#define ShadeRectOutline ShadeRectOutline24

#elif defined(MGDRAW_32_BIT)

#define ShadeHLine ShadeHLine32
#define ShadeVLine ShadeVLine32
#define ShadeLine ShadeLine32
#define ShadeRect ShadeRect32
#define ShadeTriangle ShadeTriangle32
#define ShadeCircle ShadeCircle32
#define ShadePolygon ShadePolygon32
#define ShadeCircleOutline ShadeCircleOutline32
#define ShadeRectOutline ShadeRectOutline32

#endif

void
ShadeHLine(SDL_Surface * sur, int x1, int x2, int y, MgColor & c1, MgColor & c2)
{
	/* we set up our own internal colors */
	MgColor sc1 = c1, sc2 = c2;
	
	/* see if points are in right order */
	if (x1 > x2)
	{
		SWAP(x1, x2, int);
		SWAP(sc1, sc2, MgColor);
	}
	
	/* set up our color values */
	int r = sc1.red << 16, g = sc1.green << 16, b = sc1.blue << 16, a = sc1.alpha << 16;

	/* calculate distance only once */
	int xdif = (x2 - x1 + 1);

	/* step values */
	int rs = ((sc2.red - sc1.red) << 16) / xdif;
	int gs = ((sc2.green - sc1.green) << 16) / xdif;
	int bs = ((sc2.blue - sc1.blue) << 16) / xdif;
	int as = ((sc2.alpha - sc1.alpha) << 16) / xdif;
		
	/* clip so we don't run if the line isn't visible */
	if (x2 < clip_x_min(sur) || x1 > clip_x_max(sur) || y < clip_y_min(sur) || y > clip_y_max(sur))
		return;	/* clipped */
	
	/* more clipping */
	if (x1 < clip_x_min(sur))
	{
		/* these have to be adjusted so that the blend is done correctly */
		r += (clip_x_min(sur) - x1) * rs;
		g += (clip_x_min(sur) - x1) * gs;
		b += (clip_x_min(sur) - x1) * bs;
		a += (clip_x_min(sur) - x1) * as;
		x1 = clip_x_min(sur);
	}
	if (x2 > clip_x_max(sur))
		x2 = clip_x_max(sur);
	
	/* draw the line */
	if (sc1.alpha == 255 && sc2.alpha == 255)	/* if both alphas are opaque */
	{
		Uint32 color;
		
		/* fancy for loop that updates colors automatically */
		for (int x = x1; x <= x2; x++, r += rs, g += gs, b += bs)
		{
			MAKE_COL(sur, color, (Uint8)(r >> 16), (Uint8)(g >> 16), (Uint8)(b >> 16));
			PutPixel(sur, x, y, color);
		}
	}
	else	/* we have to blend */
	{
		MgColor color;
		for (int x = x1; x <= x2; x++, r += rs, g += gs, b += bs, a += as)
		{
			color((Uint8)(r >> 16), (Uint8)(g >> 16), (Uint8)(b >> 16), (Uint8)(a >> 16));
			PutPixel(sur, x, y, color);
		}
	}
}

void
ShadeVLine(SDL_Surface * sur, int y1, int y2, int x, MgColor & c1, MgColor & c2)
{
	/* internal values */
	MgColor sc1 = c1, sc2 = c2;

	/* swap if needed */
	if (y1 > y2)
	{
		SWAP(y1, y2, int);
		SWAP(sc1, sc2, MgColor);
	}
	
	int r = sc1.red << 16, g = sc1.green << 16, b = sc1.blue << 16, a = sc1.alpha << 16;
	int ydif = (y2 - y1 + 1);

	int rs = ((sc2.red - sc1.red) << 16) / ydif;
	int gs = ((sc2.green - sc1.green) << 16) / ydif;
	int bs = ((sc2.blue - sc1.blue) << 16) / ydif;
	int as = ((sc2.alpha - sc1.alpha) << 16) / ydif;

	if (y2 < clip_y_min(sur) || y1 > clip_y_max(sur) || x < clip_x_min(sur) || x > clip_x_max(sur))
		return;

	if (y1 < clip_y_min(sur))
	{
		/* this requires more then just clipping the coordinated, we have to update the step value */
		r += (clip_y_min(sur) - y1) * rs;
		g += (clip_y_min(sur) - y1) * gs;
		b += (clip_y_min(sur) - y1) * bs;
		a += (clip_y_min(sur) - y1) * as;
		y1 = clip_y_min(sur);
	}
	if (y2 > clip_y_max(sur))
		y2 = clip_y_max(sur);

	if (sc1.alpha == 255 && sc2.alpha == 255)	/* both alphas opaque */
	{
		Uint32 color;
		for (int y = y1; y <= y2; y++, r += rs, g += gs, b += bs)
		{
			MAKE_COL(sur, color, (Uint8)(r >> 16), (Uint8)(g >> 16), (Uint8)(b >> 16));
			PutPixel(sur, x, y, color);
		}
	}
	else	/* blend */
	{
		MgColor color;
		for (int y = y1; y <= y2; y++, r += rs, g += gs, b += bs, a += as)
		{
			color((Uint8)(r >> 16), (Uint8)(g >> 16), (Uint8)(b >> 16), (Uint8)(a >> 16));
			PutPixel(sur, x, y, color);
		}
	}
}

void
ShadeLine(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & c1, MgColor & c2)
{
	int dx, dy, sdx, sdy, x, y, px, py, div;
	
	int rstep, gstep, bstep, astep;	// The amount of each component added at each step.
	int cr, cg, cb, ca;				// the current component value
	
	dx = x2 - x1;
	dy = y2 - y1;
	sdx = (dx < 0) ? -1 : 1;
	sdy = (dy < 0) ? -1 : 1;
	dx = sdx * dx + 1;
	dy = sdy * dy + 1;
	x = y = 0;
	px = x1; py = y1;

	div = (dx > dy) ? dy : dx;
	
	rstep = ((c2.red - c1.red) << 16) / div;
	gstep = ((c2.green - c1.green) << 16) / div;
	bstep = ((c2.blue - c1.blue) << 16) / div;
	astep = ((c2.alpha - c1.alpha) << 16) / div;

	cr = c1.red << 16;
	cg = c1.green << 16;
	cb = c1.blue << 16;
	ca = c1.alpha << 16;
	
	if (c1.alpha == 255 && c2.alpha == 255)
	{
		Uint32 pixel;
//		MAKE_COL(sur, pixel, col.red, col.green, col.blue);
		MAKE_COL(sur, pixel, (Uint8)(cr >> 16), (Uint8)(cg >> 16), (Uint8)(cb >> 16));	// no alpha needed since we're not blending
		
		if (dx >= dy)
		{
			for (; x < dx; x++)
			{
				PutPixel(sur, px, py, pixel);
				y += dy;
				if (y >= dx)
				{
					y -= dx;
					py += sdy;
					cr += rstep;
					cg += gstep;
					cb += bstep;
					MAKE_COL(sur, pixel, (Uint8)(cr >> 16), (Uint8)(cg >> 16), (Uint8)(cb >> 16));	// update our pixel value
				}
				px += sdx;
			}
		}
		else
		{
			for (; y < dy; y++)
			{
				PutPixel(sur, px, py, pixel);
				x += dx;
				if (x >= dy)
				{
					x -= dy;
					px += sdx;
					cr += rstep;
					cg += gstep;
					cb += bstep;
					MAKE_COL(sur, pixel, (Uint8)(cr >> 16), (Uint8)(cg >> 16), (Uint8)(cb >> 16));
				}
				py += sdy;
			}
		}
	}
	else
	{
		/* We use an internal MgColor to hold the current color. */
		MgColor curCol((Uint8)(cr >> 16), (Uint8)(cg >> 16), (Uint8)(cb >> 16), (Uint8)(ca >> 16));
		
		if (dx >= dy)
		{
			for (; x < dx; x++)
			{
				PutPixel(sur, px, py, curCol);
				y += dy;
				if (y >= dx)
				{
					y -= dx;
					py += sdy;
					cr += rstep;
					cg += gstep;
					cb += bstep;
					ca += astep;
					curCol((Uint8)(cr >> 16), (Uint8)(cg >> 16), (Uint8)(cb >> 16), (Uint8)(ca >> 16));
				}
				px += sdx;
			}
		}
		else
		{
			for (; y < dy; y++)
			{
				PutPixel(sur, px, py, curCol);
				x += dx;
				if (x >= dy)
				{
					x -= dy;
					px += sdx;
					cr += rstep;
					cg += gstep;
					cb += bstep;
					ca += astep;
					curCol((Uint8)(cr >> 16), (Uint8)(cg >> 16), (Uint8)(cb >> 16), (Uint8)(ca >> 16));
				}
				py += sdy;
			}
		}
	}
}

// blended rect (from paragui)
void
ShadeRect(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & ul, MgColor & ur, MgColor & dl, MgColor & dr)
{
	double v[2][4];
	double w = x2 - x1 + 1;
	double h = y2 - y1 + 1;
	MgColor c;
	
	v[0][0] = (ur.red - ul.red) / w;
	v[0][1] = (ur.green - ul.green) / w;
	v[0][2] = (ur.blue - ul.blue) / w;
	v[0][3] = (ur.alpha - ul.alpha) / w;
	
	v[1][0] = (dr.red - dl.red) / w;
	v[1][1] = (dr.green - dl.green) / w;
	v[1][2] = (dr.blue - dl.blue) / w;
	v[1][3] = (dr.alpha - dl.alpha) / w;
	
	int r1, g1, b1, a1, r2, g2, b2, a2;
	double yr, yg, yb, ya;
	
	for (int x = 0; x < x2 - x1 + 1; x++)
	{
		r1 = (int)(ul.red + v[0][0] * x);
		g1 = (int)(ul.green + v[0][1] * x);
		b1 = (int)(ul.blue + v[0][2] * x);
		a1 = (int)(ul.alpha + v[0][3] * x);
		
		r2 = (int)(dl.red + v[1][0] * x);
		g2 = (int)(dl.green + v[1][1] * x);
		b2 = (int)(dl.blue + v[1][2] * x);
		a2 = (int)(dl.alpha + v[1][3] * x);
		
		yr = ((double)(r2 - r1)) / h;
		yg = ((double)(g2 - g1)) / h;
		yb = ((double)(b2 - b1)) / h;
		ya = ((double)(a2 - a1)) / h;
		
		for (int y = 0; y < y2 - y1 + 1; y++)
		{
			c.red = (Uint8)(r1 + yr * y);
			c.green = (Uint8)(g1 + yg * y);
			c.blue = (Uint8)(b1 + yb * y);
			c.alpha = (Uint8)(a1 + ya * y);
			
			PutPixel(sur, x1 + x, y1 + y, c);
		}
	}
}

void
ShadeTriangle(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & c1, MgColor & c2, MgColor & c3)
{
	/* is it a valid triangle? */
	if (y1 == y3)
		return;	/* apparently not */
	
	int y;
	
	/* our internal colors */
	MgColor sc1 = c1, sc2 = c2, sc3 = c3;
	
	/* swap points */
	if (y1 > y2)
	{
		SWAP(y1, y2, int);
		SWAP(x1, x2, int);
		SWAP(sc1, sc2, MgColor);
	}
	if (y2 > y3)
	{
		SWAP(y2, y3, int);
		SWAP(x2, x3, int);
		SWAP(sc2, sc3, MgColor);
	}
	if (y1 > y2)
	{
		SWAP(y1, y2, int);
		SWAP(x1, x2, int);
		SWAP(sc1, sc2, MgColor);
	}
	
	/* starting points for the 3 lines */
	int xa = x1 << 16;
	int xb = xa;
	int xc = x2 << 16;
	
	/* starting colors */
	int r1 = (sc1.red << 16), r2 = r1, r3 = (sc2.red << 16);
	int g1 = (sc1.green << 16), g2 = g1, g3 = (sc2.green << 16);
	int b1 = (sc1.blue << 16), b2 = b1, b3 = (sc2.blue << 16);
	int a1 = (sc1.alpha << 16), a2 = a1, a3 = (sc2.alpha << 16);
	
	/* line step values */
	int m1 = 0, m2 = ((x3 - x1) << 16) / (y3 - y1), m3 = 0;
	
	/* color step values */
	int rs1 = 0, rs2 = ((sc3.red - sc1.red) << 16) / (y3 - y1), rs3 = 0;
	int gs1 = 0, gs2 = ((sc3.green - sc1.green) << 16) / (y3 - y1), gs3 = 0;
	int bs1 = 0, bs2 = ((sc3.blue - sc1.blue) << 16) / (y3 - y1), bs3 = 0;
	int as1 = 0, as2 = ((sc3.alpha - sc1.alpha) << 16) / (y3 - y1), as3 = 0;
	
	/* drawing colors */
	MgColor d1, d2;
	
	if (sc1.alpha == 255 && sc2.alpha == 255 && sc3.alpha == 255)	/* no need for blending */
	{
		/* we do not need to worry about any alpha here, which removes the alpha calculation */
		
		/* upper half */
		if (y1 == y2)
		{
			SHADE_H_LINE(sur, x1, x2, y1, sc1, sc2);
		}
		else
		{
			m1 = ((x2 - x1) << 16) / (y2 - y1);
			rs1 = ((sc2.red - sc1.red) << 16) / (y2 - y1);
			gs1 = ((sc2.green - sc1.green) << 16) / (y2 - y1);
			bs1 = ((sc2.blue - sc1.blue) << 16) / (y2 - y1);
			/* draw the line */
			for (y = y1; y <= y2; y++)
			{
				/* set the colors */
				d1((Uint8)(r1 >> 16), (Uint8)(g1 >> 16), (Uint8)(b1 >> 16));
				d2((Uint8)(r2 >> 16), (Uint8)(g2 >> 16), (Uint8)(b2 >> 16));
				
				/* shade */
				//px1 = (int)xa, px2 = (int)xb;
				SHADE_H_LINE(sur, xa  >> 16, xb >> 16, y, d1, d2);
				/* update values */
				xa += m1;
				xb += m2;
				r1 += rs1;
				g1 += gs1;
				b1 += bs1;
				r2 += rs2;
				g2 += gs2;
				b2 += bs2;
			}
		}
		
		/* lower half */
		if (y2 == y3)
		{
			SHADE_H_LINE(sur, x2, x3, y2, sc2, sc3);
		}
		else
		{
			m3 = ((x3 - x2) << 16) / (y3 - y2);
			rs3 = ((sc3.red - sc2.red) << 16) / (y3 - y2);
			gs3 = ((sc3.green - sc2.green) << 16) / (y3 - y2);
			bs3 = ((sc3.blue - sc2.blue) << 16) / (y3 - y2);
			
			/* draw the lines */
			for (y = y2 + 1; y <= y3; y++)
			{
				d1((Uint8)(r2 >> 16), (Uint8)(g2 >> 16), (Uint8)(b2 >> 16));
				d2((Uint8)(r3 >> 16), (Uint8)(g3 >> 16), (Uint8)(b3 >> 16));
				
//				px1 = (int)xb, px2 = (int)xc;
				SHADE_H_LINE(sur, xb >> 16, xc >> 16, y, d1, d2);
				
				xb += m2;
				xc += m3;
				r2 += rs2;
				g2 += gs2;
				b2 += bs2;
				r3 += rs3;
				g3 += gs3;
				b3 += bs3;
			}
		}
	}
	else
	{
		/* blend here */
		
		/* upper half */
		if (y1 == y2)
		{
			SHADE_H_LINE(sur, x1, x2, y1, sc1, sc2);
		}
		else
		{
			m1 = ((x2 - x1) << 16) / (y2 - y1);
			rs1 = ((sc2.red - sc1.red) << 16) / (y2 - y1);
			gs1 = ((sc2.green - sc1.green) << 16) / (y2 - y1);
			bs1 = ((sc2.blue - sc1.blue) << 16) / (y2 - y1);
			as1 = ((sc2.alpha - sc1.alpha) << 16) / (y2 - y1);
			
			/* draw the line */
			for (y = y1; y <= y2; y++)
			{
				/* set the colors */
				d1((Uint8)(r1 >> 16), (Uint8)(g1 >> 16), (Uint8)(b1 >> 16), (Uint8)(a1 >> 16));
				d2((Uint8)(r2 >> 16), (Uint8)(g2 >> 16), (Uint8)(b2 >> 16), (Uint8)(a2 >> 16));
				
				/* shade */
				//px1 = (int)xa, px2 = (int)xb;
				SHADE_H_LINE(sur, (xa  >> 16), (xb >> 16), y, d1, d2);
				/* update values */
				xa += m1;
				xb += m2;
				
				r1 += rs1;
				g1 += gs1;
				b1 += bs1;
				a1 += as1;
				
				r2 += rs2;
				g2 += gs2;
				b2 += bs2;
				a2 += as2;
			}
		}
		
		/* lower half */
		if (y2 == y3)
		{
			SHADE_H_LINE(sur, x2, x3, y2, sc2, sc3);
		}
		else
		{
			m3 = ((x3 - x2) << 16) / (y3 - y2);
			
			rs3 = ((sc3.red - sc2.red) << 16) / (y3 - y2);
			gs3 = ((sc3.green - sc2.green) << 16) / (y3 - y2);
			bs3 = ((sc3.blue - sc2.blue) << 16) / (y3 - y2);
			as3 = ((sc3.alpha - sc2.alpha) << 16) / (y3 - y2);
			
			/* draw the lines */
			for (y = y2 + 1; y <= y3; y++)
			{
				d1((Uint8)(r2 >> 16), (Uint8)(g2 >> 16), (Uint8)(b2 >> 16), (Uint8)(a2 >> 16));
				d2((Uint8)(r3 >> 16), (Uint8)(g3 >> 16), (Uint8)(b3 >> 16), (Uint8)(a3 >> 16));
				
//				px1 = (int)xb, px2 = (int)xc;
				SHADE_H_LINE(sur, (xb >> 16), (xc >> 16), y, d1, d2);
				
				xb += m2;
				xc += m3;
				
				r2 += rs2;
				g2 += gs2;
				b2 += bs2;
				a2 += as2;
				
				r3 += rs3;
				g3 += gs3;
				b3 += bs3;
				a3 += as3;
			}
		}
	}
}

void
ShadeCircle(SDL_Surface * sur, int x, int y, int radius, MgColor & center, MgColor & outside)
{
	/* this routine is just like FillCircle(), except we use ShadeHLine() to create the blend */
	int x1, y1, x2, y2;
	int cx = 0, cy = radius, ocx = 0xFFFFFFFF, ocy = 0xFFFFFFFF, df = 1 - radius, de = 3, dse = -2 * radius + 5;
	int xpcx, xmcx, xpcy, xmcy, ypcy, ymcy, ypcx, ymcx;
	int middlex = x;

	MgColor topColor(outside);	/* this is used as the starting color for the "top" part of the
									circle that is drawn */
	
	float r = (float)center.red, g = (float)center.green, b = (float)center.blue, a = (float)center.alpha;
	float tr = (float)outside.red, tg = (float)outside.green, tb = (float)outside.blue, ta = (float)outside.alpha;
	
	float rs = (r - (float)outside.red) / (float)radius,
		  gs = (g - (float)outside.green) / (float)radius,
		  bs = (b - (float)outside.blue) / (float)radius,
		  as = (a - (float)outside.alpha) / (float)radius;
 
 	//printf("%.2f, %.2f, %.2f, %.2f\n", rs, gs, bs, as);
 	
	if (center.alpha == 255 && outside.alpha == 255)
	{

		x1 = x - radius;
		y1 = y - radius;
		x2 = x + radius;
		y2 = y + radius;
		
		do
		{
			xpcx = x + cx;
			xmcx = x - cx;
			xpcy = x + cy;
			xmcy = x - cy;
			
			if (ocy != cy)
			{
				if (cy > 0)
				{
					ypcy = y + cy;
					ymcy = y - cy;
					SHADE_H_LINE(sur, xmcx, middlex, ypcy, outside, topColor);
					SHADE_H_LINE(sur, middlex + 1, xpcx, ypcy, topColor, outside);
					
					SHADE_H_LINE(sur, xmcx, middlex, ymcy, outside, topColor);
					SHADE_H_LINE(sur, middlex + 1, xpcx, ymcy, topColor, outside);
				}
				else
				{
					SHADE_H_LINE(sur, xmcx, middlex, y, outside, topColor);
					SHADE_H_LINE(sur, middlex + 1, xpcx, y, topColor, outside);
				}
				ocy = cy;
			}
			if (ocx != cx)
			{
				if (cx != cy)
				{
					if (cx > 0)
					{
						ypcx = y + cx;
						ymcx = y - cx;
						SHADE_H_LINE(sur, xmcy, middlex, ymcx, outside, center);
						SHADE_H_LINE(sur, middlex + 1, xpcy, ymcx, center, outside);
							
						SHADE_H_LINE(sur, xmcy, middlex, ypcx, outside, center);
						SHADE_H_LINE(sur, middlex + 1, xpcy, ypcx, center, outside);
					}
					else
					{
						SHADE_H_LINE(sur, xmcy, middlex, y, outside, center);
						SHADE_H_LINE(sur, middlex + 1, xpcy, y, center, outside);
					}
				}
				ocx = cx;
			}
			
			if (df < 0)
			{
				df += de;
				de += 2;
				dse += 2;
			}
			else
			{
				df += dse;
				de += 2;
				dse += 4;
				cy--;
				/* update our y-based colors */
				tr += rs;
				tg += gs;
				tb += bs;
				topColor((Uint8)tr, (Uint8)tg, (Uint8)tb);
			}
			cx++;
			/* update our x based colors */
			r -= rs;
			g -= gs;
			b -= bs;
			center((Uint8)r, (Uint8)g, (Uint8)b);
		} while (cx <= cy);
	}
	else
	{
		/* we need to do alpha-blending */
		
		x1 = x - radius;
		y1 = y - radius;
		x2 = x + radius;
		y2 = y + radius;
		
		do
		{
			xpcx = x + cx;
			xmcx = x - cx;
			xpcy = x + cy;
			xmcy = x - cy;
			
			if (ocy != cy)
			{
				if (cy > 0)
				{
					ypcy = y + cy;
					ymcy = y - cy;
					SHADE_H_LINE(sur, xmcx, middlex, ypcy, outside, topColor);
					SHADE_H_LINE(sur, middlex + 1, xpcx, ypcy, topColor, outside);
					
					SHADE_H_LINE(sur, xmcx, middlex, ymcy, outside, topColor);
					SHADE_H_LINE(sur, middlex + 1, xpcx, ymcy, topColor, outside);
				}
				else
				{
					SHADE_H_LINE(sur, xmcx, middlex, y, outside, topColor);
					SHADE_H_LINE(sur, middlex + 1, xpcx, y, topColor, outside);
				}
				ocy = cy;
			}
			if (ocx != cx)
			{
				if (cx != cy)
				{
					if (cx > 0)
					{
						ypcx = y + cx;
						ymcx = y - cx;
						SHADE_H_LINE(sur, xmcy, middlex, ymcx, outside, center);
						SHADE_H_LINE(sur, middlex + 1, xpcy, ymcx, center, outside);
							
						SHADE_H_LINE(sur, xmcy, middlex, ypcx, outside, center);
						SHADE_H_LINE(sur, middlex + 1, xpcy, ypcx, center, outside);
					}
					else
					{
						SHADE_H_LINE(sur, xmcy, middlex, y, outside, center);
						SHADE_H_LINE(sur, middlex + 1, xpcy, y, center, outside);
					}
				}
				ocx = cx;
			}
			
			if (df < 0)
			{
				df += de;
				de += 2;
				dse += 2;
			}
			else
			{
				df += dse;
				de += 2;
				dse += 4;
				cy--;
				/* update our y-based colors */
				tr += rs;
				tg += gs;
				tb += bs;
				ta += as;
				topColor((Uint8)tr, (Uint8)tg, (Uint8)tb, (Uint8)ta);
			}
			cx++;
			/* update our x based colors */
			r -= rs;
			g -= gs;
			b -= bs;
			a -= as;
			center((Uint8)r, (Uint8)g, (Uint8)b, (Uint8)a);
		} while (cx <= cy);
	}	
}

void
ShadeCircleOutline(SDL_Surface * sur, int x, int y, int r, MgColor & ctop, MgColor & cright, MgColor &
					cbottom, MgColor & cleft)
{
	int cx = 0;
	int cy = r;
	int ocx = 0xFFFFFFFF;
	int ocy = 0xFFFFFFFF;
	int df = 1 - r;
	int de = 3;
	int dse = -2 * r + 5;
	int xpcx, xmcx, xpcy, xmcy;
	int ypcy, ymcy, ypcx, ymcx;
	
	/* sanity check */
	if (r < 0 || r == 0)	/* no point in drawing negative circles or circles that are just dots */
		return;
	
	/* normally, i would not use underscores, but in this case... */
	int ttol_r, ttol_g, ttol_b, ttol_a;
	int ttor_r, ttor_g, ttor_b, ttor_a;
	int rtot_r, rtot_g, rtot_b, rtot_a;
	int rtob_r, rtob_g, rtob_b, rtob_a;
	int btor_r, btor_g, btor_b, btor_a;
	int btol_r, btol_g, btol_b, btol_a;
	int ltot_r, ltot_g, ltot_b, ltot_a;
	int ltob_r, ltob_g, ltob_b, ltob_a;
	
	/* top going to left and right */
	ttol_r = ttor_r = ctop.red << 16;
	ttol_g = ttor_g = ctop.green << 16;
	ttol_b = ttor_b = ctop.blue << 16;
	ttol_a = ttor_a = ctop.alpha << 16;
	
	/* right goint to top and bottom */
	rtot_r = rtob_r = cright.red << 16;
	rtot_g = rtob_g = cright.green << 16;
	rtot_b = rtob_b = cright.blue << 16;
	rtot_a = rtob_a = cright.alpha << 16;
	
	/* bottom goint to right and left */
	btor_r = btol_r = cbottom.red << 16;
	btor_g = btol_g = cbottom.green << 16;
	btor_b = btol_b = cbottom.blue << 16;
	btor_a = btol_a = cbottom.alpha << 16;
	
	/* left going to top and bottom */
	ltot_r = ltob_r = cleft.red << 16;
	ltot_g = ltob_g = cleft.green << 16;
	ltot_b = ltob_b = cleft.blue << 16;
	ltot_a = ltob_a = cleft.alpha << 16;
	
	/* step values */
	int left_top_rs, left_top_gs, left_top_bs, left_top_as;
	int right_top_rs, right_top_gs, right_top_bs, right_top_as;
	int right_bottom_rs, right_bottom_gs, right_bottom_bs, right_bottom_as;
	int left_bottom_rs, left_bottom_gs, left_bottom_bs, left_bottom_as;
	
	left_top_rs = ((cleft.red - ctop.red) << 16) / r;
	left_top_gs = ((cleft.green - ctop.green) << 16) / r;
	left_top_bs = ((cleft.blue - ctop.blue) << 16) / r;
	left_top_as = ((cleft.alpha - ctop.alpha) << 16) / r;
	
	right_top_rs = ((cright.red - ctop.red) << 16) / r;
	right_top_gs = ((cright.green - ctop.green) << 16) / r;
	right_top_bs = ((cright.blue - ctop.blue) << 16) / r;
	right_top_as = ((cright.alpha - ctop.alpha) << 16) / r;

	right_bottom_rs = ((cbottom.red - cright.red) << 16) / r;
	right_bottom_gs = ((cbottom.green - cright.green) << 16) / r;
	right_bottom_bs = ((cbottom.blue - cright.blue) << 16) / r;
	right_bottom_as = ((cbottom.alpha - cright.alpha) << 16) / r;
	
	left_bottom_rs = ((cbottom.red - cleft.red) << 16) / r;
	left_bottom_gs = ((cbottom.green - cleft.green) << 16) / r;
	left_bottom_bs = ((cbottom.blue - cleft.blue) << 16) / r;
	left_bottom_as = ((cbottom.alpha - cleft.alpha) << 16) / r;
	
	MgColor color(255, 255, 255);;
	/* the draw loop */
	do
	{
		if ((ocy != cy) || (ocx != cx))
		{
			xpcx = x + cx;
			xmcx = x - cx;
			if (cy > 0)
			{
				ypcy = y + cy;
				ymcy = y - cy;
				
				color(btol_r >> 16, btol_g >> 16, btol_b >> 16, btol_a >> 16);
				PutPixel(sur, xmcx, ypcy, color);
				
				color(btor_r >> 16, btor_g >> 16, btor_b >> 16, btor_a >> 16);
				PutPixel(sur, xpcx, ypcy, color);
				
				color(ttol_r >> 16, ttol_g >> 16, ttol_b >> 16, ttol_a >> 16);
				PutPixel(sur, xmcx, ymcy, color);
				
				color(ttor_r >> 16, ttor_g >> 16, ttor_b >> 16, ttor_a >> 16);
				PutPixel(sur, xpcx, ymcy, color);
			}
			else
			{
				/*
				PutPixel(sur, xmcx, y, color);
				PutPixel(sur, xpcx, y, color);*/
			}
			
			ocy = cy;
			xpcy = x + cy;
			xmcy = x - cy;
			
			if (cx > 0)
			{
				ypcx = y + cx;
				ymcx = y - cx;
				
				color(ltob_r >> 16, ltob_g >> 16, ltob_b >> 16, ltob_a >> 16);
				PutPixel(sur, xmcy, ypcx, color);
				color(ltot_r >> 16, ltot_g >> 16, ltot_b >> 16, ltot_a >> 16);
				PutPixel(sur, xmcy, ymcx, color);

				color(rtob_r >> 16, rtob_g >> 16, rtob_b >> 16, rtob_a >> 16);
				PutPixel(sur, xpcy, ypcx, color);
				color(rtot_r >> 16, rtot_g >> 16, rtot_b >> 16, rtot_a >> 16);
				PutPixel(sur, xpcy, ymcx, color);
			}
			else
			{
				//PutPixel(sur, xmcy, y, color);
				//PutPixel(sur, xpcy, y, color);
			}
			
			ocx = cx;
		}
		
		/* update */
		if (df < 0)
		{
			df += de;
			de += 2;
			dse += 2;
		}
		else
		{
			df += dse;
			de += 2;
			dse += 4;
			cy--;
			
			/* update y based values */
			ltot_r -= left_top_rs;
			ltot_g -= left_top_gs;
			ltot_b -= left_top_bs;
			ltot_a -= left_top_as;
			
			rtot_r -= right_top_rs;
			rtot_g -= right_top_gs;
			rtot_b -= right_top_bs;
			rtot_a -= right_top_as;
			
			rtob_r += right_bottom_rs;
			rtob_g += right_bottom_gs;
			rtob_b += right_bottom_bs;
			rtob_a += right_bottom_as;
			
			ltob_r += left_bottom_rs;
			ltob_g += left_bottom_gs;
			ltob_b += left_bottom_bs;
			ltob_a += left_bottom_as;
		}
		/* update x based values */
		ttol_r += left_top_rs;
		ttol_g += left_top_gs;
		ttol_b += left_top_bs;
		ttol_a += left_top_as;
		
		ttor_r += right_top_rs;
		ttor_g += right_top_gs;
		ttor_b += right_top_bs;
		ttor_a += right_top_as;
		
		btor_r -= right_bottom_rs;
		btor_g -= right_bottom_gs;
		btor_b -= right_bottom_bs;
		btor_a -= right_bottom_as;
		
		btol_r -= left_bottom_rs;
		btol_g -= left_bottom_gs;
		btol_b -= left_bottom_bs;
		btol_a -= left_bottom_as;
		
		cx++;
	} while (cx <= cy);
}

void
ShadeRectOutline(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & topleft, MgColor & topright,
					MgColor & bottomright, MgColor & bottomleft)
{
	SHADE_V_LINE(sur, y1, y2, x1, topleft, bottomleft);
	SHADE_V_LINE(sur, y1, y2, x2, topright, bottomright);
	SHADE_H_LINE(sur, x1, x2, y1, topleft, topright);
	SHADE_H_LINE(sur, x1, x2, y2, bottomleft, bottomright);
}

/* convex polygons only!!!!!! */
void
ShadePolygon(SDL_Surface * sur, MgIVertex * vertices, int n)
{
	int i;
	
	for (i = 0; (i + 2) < n ; i++)
	{
		ShadeTriangle(sur, vertices[0].x, vertices[0].y, vertices[i + 1].x,
						vertices[i + 1].y, vertices[i + 2].x, vertices[i + 2].y,
						vertices[0].c, vertices[i + 1].c, vertices[i + 2].c);
	}
}

#undef ShadeCircle		
#undef ShadeHLine
#undef ShadeVLine
#undef ShadeLine
#undef ShadeRect
#undef ShadeTriangle
#undef ShadePolygon
#undef ShadeCircleOutline
#undef ShadeRectOutline

#endif

#endif

