//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

/* this file defines all the functions that draw outlined shapes */
#ifdef DO_FUNCS

#if defined(MGDRAW_8_BIT)

#define StrokeHLine StrokeHLine8
#define StrokeVLine StrokeVLine8
#define StrokeLine StrokeLine8
#define StrokeRect StrokeRect8
#define StrokeCircle StrokeCircle8
#define StrokeEllipse StrokeEllipse8
#define StrokeTriangle StrokeTriangle8
#define StrokePolygon StrokePolygon8

#elif defined(MGDRAW_16_BIT)

#define StrokeHLine StrokeHLine16
#define StrokeVLine StrokeVLine16
#define StrokeLine StrokeLine16
#define StrokeRect StrokeRect16
#define StrokeCircle StrokeCircle16
#define StrokeEllipse StrokeEllipse16
#define StrokeTriangle StrokeTriangle16
#define StrokePolygon StrokePolygon16

#elif defined(MGDRAW_24_BIT)

#define StrokeHLine StrokeHLine24
#define StrokeVLine StrokeVLine24
#define StrokeLine StrokeLine24
#define StrokeRect StrokeRect24
#define StrokeCircle StrokeCircle24
#define StrokeEllipse StrokeEllipse24
#define StrokeTriangle StrokeTriangle24
#define StrokePolygon StrokePolygon24

#elif defined(MGDRAW_32_BIT)

#define StrokeHLine StrokeHLine32
#define StrokeVLine StrokeVLine32
#define StrokeLine StrokeLine32
#define StrokeRect StrokeRect32
#define StrokeCircle StrokeCircle32
#define StrokeEllipse StrokeEllipse32
#define StrokeTriangle StrokeTriangle32
#define StrokePolygon StrokePolygon32

#endif

void
StrokeHLine(SDL_Surface * sur, int x1, int x2, int y, MgColor & col)
{
	if (x1 > x2)
		SWAP(x1, x2, int)
	
	if (col.alpha == 255) 	// no blend
	{
		Uint32 pixel;
		MAKE_COL(sur, pixel, col.red, col.green, col.blue);
		
		for (int x = x1; x <= x2; x++)
			PutPixel(sur, x, y, pixel);
	}
	else
	{
		for (int x = x1; x <= x2; x++)
			PutPixel(sur, x, y, col);
	}
}

void
StrokeVLine(SDL_Surface * sur, int y1, int y2, int x, MgColor & col)
{
	if (y1 > y2)
		SWAP(y1, y2, int)
		
	if (col.alpha == 255)
	{
		Uint32 pixel;
		MAKE_COL(sur, pixel, col.red, col.green, col.blue);
		
		for (int y = y1; y <= y2; y++)
			PutPixel(sur, x, y, pixel);
	}
	else
	{
		for (int y = y1; y <= y2; y++)
			PutPixel(sur, x, y, col);
	}
}

void
StrokeLine(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col)
{
	int dx, dy, sdx, sdy, x, y, px, py;
	
	dx = x2 - x1;
	dy = y2 - y1;
	sdx = (dx < 0) ? -1 : 1;
	sdy = (dy < 0) ? -1 : 1;
	dx = sdx * dx + 1;
	dy = sdy * dy + 1;
	x = y = 0;
	px = x1; py = y1;

	if (col.alpha == 255)
	{
		Uint32 pixel;
		MAKE_COL(sur, pixel, col.red, col.green, col.blue);
		
		if (dx >= dy)
		{
			for (; x < dx; x++)
			{
				PutPixel(sur, px, py, pixel);
				y += dy;
				if (y >= dx)
				{
					y -= dx;
					py += sdy;
				}
				px += sdx;
			}
		}
		else
		{
			for (; y < dy; y++)
			{
				PutPixel(sur, px, py, pixel);
				x += dx;
				if (x >= dy)
				{
					x -= dy;
					px += sdx;
				}
				py += sdy;
			}
		}
	}
	else
	{
		if (dx >= dy)
		{
			for (; x < dx; x++)
			{
				PutPixel(sur, px, py, col);
				y += dy;
				if (y >= dx)
				{
					y -= dx;
					py += sdy;
				}
				px += sdx;
			}
		}
		else
		{
			for (; y < dy; y++)
			{
				PutPixel(sur, px, py, col);
				x += dx;
				if (x >= dy)
				{
					x -= dy;
					px += sdx;
				}
				py += sdy;
			}
		}
	}
}

void
StrokeRect(SDL_Surface * sur, int x1, int y1, int x2, int y2, MgColor & col)
{
	int x, y;
	
	if (col.alpha == 255)
	{
		Uint32 pixel;
		MAKE_COL(sur, pixel, col.red, col.green, col.blue);

		for (x = x1; x <= x2; x++)
		{
			PutPixel(sur, x, y1, pixel);
			PutPixel(sur, x, y2, pixel);
		}
		for (y = y1; y <= y2; y++)
		{
			PutPixel(sur, x1, y, pixel);
			PutPixel(sur, x2, y, pixel);
		}
	}
	else
	{
		for (x = x1; x <= x2; x++)
		{
			PutPixel(sur, x, y1, col);
			PutPixel(sur, x, y2, col);
		}
		for (y = y1; y <= y2; y++)
		{
			PutPixel(sur, x1, y, col);
			PutPixel(sur, x2, y, col);
		}
	}
}

void
StrokeCircle(SDL_Surface * sur, int x, int y, int radius, MgColor & col)
{
	int cx = 0, cy = radius, df = 1 - radius, de = 3, dse = -2 * radius + 5;

	if (col.alpha == 255)
	{
		Uint32 pixel;
		MAKE_COL(sur, pixel, col.red, col.green, col.blue);

		do
		{
			PutPixel(sur, x + cx, y + cy, pixel);
			PutPixel(sur, x - cx, y + cy, pixel);
			PutPixel(sur, x + cx, y - cy, pixel);
			PutPixel(sur, x - cx, y - cy, pixel);
			PutPixel(sur, x + cy, y + cx, pixel);
			PutPixel(sur, x + cy, y - cx, pixel);
			PutPixel(sur, x - cy, y + cx, pixel);
			PutPixel(sur, x - cy, y - cx, pixel);
			if (df < 0)
			{
				df += de;
				de += 2;
				dse += 2;
			}
			else
			{
				df += dse;
				de += 2;
				dse += 4;
				cy--;
			}
			cx++;
		} while (cx <= cy);
	}
	else
	{
		do
		{
			PutPixel(sur, x + cx, y + cy, col);
			PutPixel(sur, x - cx, y + cy, col);
			PutPixel(sur, x + cx, y - cy, col);
			PutPixel(sur, x - cx, y - cy, col);
			PutPixel(sur, x + cy, y + cx, col);
			PutPixel(sur, x + cy, y - cx, col);
			PutPixel(sur, x - cy, y + cx, col);
			PutPixel(sur, x - cy, y - cx, col);
			if (df < 0)
			{
				df += de;
				de += 2;
				dse += 2;
			}
			else
			{
				df += dse;
				de += 2;
				dse += 4;
				cy--;
			}
			cx++;
		} while (cx <= cy);
	}
}

void
StrokeEllipse(SDL_Surface * sur, int x, int y, int rx, int ry, MgColor & col)
{
	int ix, iy, h, i, j, k, oh, oi, oj, ok;

	if (rx < 1) rx = 1;
	if (ry < 1) ry = 1;
	h = i = j = k = 0xFFFF;

	if (col.alpha == 255)
	{
		Uint32 pixel;
		MAKE_COL(sur, pixel, col.red, col.green, col.blue);

		if (rx > ry)
		{
			ix = 0;
			iy = rx * 64;
			do
			{
				oh = h; oi = i; oj = j; ok = k;
				h = (ix + 32) >> 6;
				i = (iy + 32) >> 6;
				j = (h * ry) / rx;
				k = (i * ry) / rx;
				if (((h != 0) || (k != ok)) && (h < oi))
				{
					PutPixel(sur, x + h, y + k, pixel);
					if (h)
						PutPixel(sur, x - h, y + k, pixel);
					if (k)
					{
						PutPixel(sur, x + h, y - k, pixel);
						if (h)
							PutPixel(sur, x - h, y - k, pixel);
					}
				}
				if (((i != oi) || (j != oj)) && (h < i))
				{
					PutPixel(sur, x + i, y + j, pixel);
					if (i)
						PutPixel(sur, x - i, y + j, pixel);
					if (j)
					{
						PutPixel(sur, x + i, y - j, pixel);
						if (i)
							PutPixel(sur, x - i, y - j, pixel);
					}
				}
				ix = ix + iy / rx;
				iy = iy - ix / rx;
			} while (i > h);
		}
		else
		{
			ix = 0;
			iy = ry * 64;
			do
			{
				oh = h; oi = i; oj = j; ok = k;
				h = (ix + 32) >> 6;
				i = (iy + 32) >> 6;
				j = (h * rx) / ry;
				k = (i * rx) / ry;
				if (((j != oj) || (i != oi)) && (h < i))
				{
					PutPixel(sur, x + j, y + i, pixel);
					if (j)
						PutPixel(sur, x - j, y + i, pixel);
					if (i)
					{
						PutPixel(sur, x + j, y - i, pixel);
						if (j)
							PutPixel(sur, x - j, y - i, pixel);
					}
				}
				if (((k != ok) || (h != oh)) && (h < oi))
				{
					PutPixel(sur, x + k, y + h, pixel);
					if (k)
						PutPixel(sur, x - k, y + h, pixel);
					if (h)
					{
						PutPixel(sur, x + k, y - h, pixel);
						if (h)
							PutPixel(sur, x - k, y - h, pixel);
					}
				}
				ix = ix + iy / ry;
				iy = iy - ix / ry;
			} while (i > h);
		}
	}
	else
	{
		if (rx > ry)
		{
			ix = 0;
			iy = rx * 64;
			do
			{
				oh = h; oi = i; oj = j; ok = k;
				h = (ix + 32) >> 6;
				i = (iy + 32) >> 6;
				j = (h * ry) / rx;
				k = (i * ry) / rx;
				if (((h != 0) || (k != ok)) && (h < oi))
				{
					PutPixel(sur, x + h, y + k, col);
					if (h)
						PutPixel(sur, x - h, y + k, col);
					if (k)
					{
						PutPixel(sur, x + h, y - k, col);
						if (h)
							PutPixel(sur, x - h, y - k, col);
					}
				}
				if (((i != oi) || (j != oj)) && (h < i))
				{
					PutPixel(sur, x + i, y + j, col);
					if (i)
						PutPixel(sur, x - i, y + j, col);
					if (j)
					{
						PutPixel(sur, x + i, y - j, col);
						if (i)
							PutPixel(sur, x - i, y - j, col);
					}
				}
				ix = ix + iy / rx;
				iy = iy - ix / rx;
			} while (i > h);
		}
		else
		{
			ix = 0;
			iy = ry * 64;
			do
			{
				oh = h; oi = i; oj = j; ok = k;
				h = (ix + 32) >> 6;
				i = (iy + 32) >> 6;
				j = (h * rx) / ry;
				k = (i * rx) / ry;
				if (((j != oj) || (i != oi)) && (h < i))
				{
					PutPixel(sur, x + j, y + i, col);
					if (j)
						PutPixel(sur, x - j, y + i, col);
					if (i)
					{
						PutPixel(sur, x + j, y - i, col);
						if (j)
							PutPixel(sur, x - j, y - i, col);
					}
				}
				if (((k != ok) || (h != oh)) && (h < oi))
				{
					PutPixel(sur, x + k, y + h, col);
					if (k)
						PutPixel(sur, x - k, y + h, col);
					if (h)
					{
						PutPixel(sur, x + k, y - h, col);
						if (h)
							PutPixel(sur, x - k, y - h, col);
					}
				}
				ix = ix + iy / ry;
				iy = iy - ix / ry;
			} while (i > h);
		}
	}
}

void
StrokeTriangle(SDL_Surface * sur, int x1, int y1, int x2, int y2, int x3, int y3, MgColor & col)
{
	StrokeLine(sur, x1, y1, x2, y2, col);
	StrokeLine(sur, x2, y2, x3, y3, col);
	StrokeLine(sur, x3, y3, x1, y1, col);
}

/* this function is from SDL_gfxPrimitives, but it was modified to work
 * with libMgTk
 */
void
StrokePolygon(SDL_Surface * sur, MgIVertex * vertices, int numVertices)
{
	MgIVertex * v1, * v2;
	
	/* make sure we're a polygon*/
	if (numVertices < 3)
		return;
	
	/* vertex pointer setup */
	v1 = v2 = vertices;
	v2++;
	
	/* time to draw it */
	/* I didn't split this function up into alpha and non-alpha parts because each vertex
	 * can have a different color, which means it can also have different alpha values
	 */
	for (int i = 1; i < numVertices; i++)
	{
		/* we draw a line from the first vertex to the second vertex using the
		 * first vertexes color. This allows for some interestingly colored
		 * polygons
		 */
		STROKE_LINE(sur, v1->x, v1->y, v2->x, v2->y, v1->c);
		/* update vertex pointers */
		v1 = v2;
		v2++;
	}
	/* draw the closing line */
	STROKE_LINE(sur, v1->x, v1->y, vertices->x, vertices->y, v1->c);
}

#undef StrokeHLine
#undef StrokeVLine
#undef StrokeLine
#undef StrokeRect
#undef StrokeCircle
#undef StrokeEllipse
#undef StrokeTriangle
#undef StrokePolygon

#endif
