// MgSubSytem.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGSUBSYSTEM_H
#define MGSUBSYSTEM_H

/*! \file MgSubSystem.h
	Contains the abstract MgSubSystem class.
*/

#include "MgBeginCode.h"

/*!	All systems derive from this class. */
class MGDLL MgSubSystem
{
public:
	//! Constructor. Does nothing.
	MgSubSystem() {}
	//! Destructor. Does nothing.
	virtual ~MgSubSystem() {}
	
	/*!	Checks to see if the subsystem has been initialized.
		
		\retval true It has.
		\retval fales It has not.
	*/
	virtual bool IsRunning() const = 0;

	/*!	Initializes the subsytem if it is not running already.
	
		\retval true Initialized.
		\retval false Error.
	*/
	virtual bool Init() = 0;
	
	/*!	Quit the subsystem if it is not running. */
	virtual void Quit() = 0;

private:
};

#include "MgEndCode.h"

#endif

