// MgTCPNet.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include "MgTCPNet.h"

#include "MgNetLib.h"
#include "debug_new.h"

MgTCPNet::MgTCPNet(void)
{
//	ipAddress = NULL;
	MgNet_TCP_Close(tcpSock);
  	tcpSock = NULL;
}

MgTCPNet::MgTCPNet(int maxSocketSet)
{
	sockSet = MgNet_AllocSocketSet(maxSocketSet);
	
	if(sockSet == NULL)
	{
    		throw /*something*/;
  	}
  	
//  	ipAddress = NULL;
  	tcpSock = NULL;
}

MgTCPNet::~MgTCPNet(void)
{
	if(tcpSock != NULL)
	{
		MgNet_TCP_Close(tcpSock);
	}
	if(sockSet != NULL)
	{
		MgNet_FreeSocketSet(sockSet);
	}
	
	tcpSock = NULL;
	sockSet = NULL;
}

bool MgTCPNet::Open(Uint16 port)
{	
	if(MgNet::ResolveHost(&ipAddress, NULL, port) == 0)
	{
		return Open(sockSet,&ipAddress);
	}
	return false;
}

bool MgTCPNet::Open(Uint16 port,char *host)
{	
	if(MgNet::ResolveHost(&ipAddress, host, port) == 0)
	{
		return Open(sockSet,&ipAddress);
	}
	
	return false;
}

bool MgTCPNet::Open(MgNetIPAddress *ip)
{
	return Open(sockSet,ip);
}

bool MgTCPNet::Open(MgNetSocketSet *socketSet, char * host, Uint16 port)
{
	if(MgNet::ResolveHost(&ipAddress, host, port) == 0)
	{
		return Open(socketSet,&ipAddress);
	}
	return false;
}

bool MgTCPNet::Open(MgNetSocketSet *socketSet,MgNetIPAddress *ip)
{
	ipAddress.port = ip->port;
	ipAddress.host = ip->host;
	
  	tcpSock = MgNet_TCP_Open(&ipAddress);
  	
  	if(tcpSock == NULL)
  	{
    	return false;
    }
 
 	MgNet_TCP_AddSocket(socketSet, tcpSock);
 
   	return true; 	
}

MgNetSocketSet *MgTCPNet::GetSocketSet(void)
{
	return sockSet;
}

bool MgTCPNet::Accept(MgNetTCPSocket *tcpIncomingSock)
{
	if(tcpSock == NULL)
	{
		return false;
	}
	
	tcpIncomingSock = MgNet_TCP_Accept(tcpSock);
	
	if(tcpIncomingSock == NULL)
	{
		return false;
	}
	
	MgNet_TCP_AddSocket(sockSet, tcpIncomingSock);
	
	return true;
}

MgNetIPAddress * MgTCPNet::GetPeerAddress(void)
{
	if(tcpSock == NULL)
	{
		return NULL;
	}
	
	return MgNet_TCP_GetPeerAddress(tcpSock);
}

bool MgTCPNet::Send(void *data,int len)
{
	if(tcpSock == NULL)
	{
		return false;
	}
	
	if(MgNet_TCP_Send(tcpSock,data,len) < len)
	{
		return false;
	}
	return true;
}

bool MgTCPNet::Receive(void *data,int maxlen)
{
	if(tcpSock == NULL)
	{
		return false;
	}
	
	if(MgNet_TCP_Recv(tcpSock,data,maxlen) <= 0)
	{
		return false;
	}
	return true;
}

int MgTCPNet::HandleClient(int (*callback)(MgNetGenericSocket *,void *), MgNetGenericSocket * sock, void * data)
{
	if(tcpSock == NULL)
	{
		return -1;
	}
	
	if(MgNet_SocketReady(tcpSock)) 
	{
		if(sock == NULL)
		{
			return callback((MgNetGenericSocket *)tcpSock,data);
		}
		else
		{
			return callback(sock,data);
		}
	}
	return -1;
}

int MgTCPNet::HandleServer(int (*callback)(MgNetGenericSocket *,void *), MgNetGenericSocket * sock, void * data)
{
	if(tcpSock == NULL)
	{
		return -1;
	}
	
	if(MgNet_SocketReady(tcpSock)) 
	{
		if(sock == NULL)
		{
			return callback((MgNetGenericSocket *)tcpSock,data);
		}
		else
		{
			return callback(sock,data);
		}
	}
	return -1;
}

bool MgTCPNet::CheckSocketSet(Uint32 timeout)
{
	if(sockSet == NULL)
	{
		return false;
	}
	
	if(MgNet_CheckSockets(sockSet,timeout) == -1)
	{
		return false;
	}
	return true;
}

void MgTCPNet::Close(void)
{
	this->~MgTCPNet();
}