// MgUDPNet.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include "MgUDPNet.h"
#include "MgNetLib.h"
#include "debug_new.h"

MgUDPNet::MgUDPNet(void)
{
//	ipAddress = NULL;
  	udpSock = NULL;
}

MgUDPNet::MgUDPNet(int maxSocketSet)
{
	sockSet = MgNet_AllocSocketSet(maxSocketSet);
	
	if(sockSet == NULL)
	{
    		throw /*something*/;
  	}
  	
//  	ipAddress = NULL;
  	udpSock = NULL;
}

MgUDPNet::~MgUDPNet(void)
{
	if(udpSock != NULL)
	{
		MgNet_UDP_Close(udpSock);
	}
	if(sockSet != NULL)
	{
		MgNet_FreeSocketSet(sockSet);
	}
	
	udpSock = NULL;
	sockSet = NULL;
}

bool MgUDPNet::Open(Uint16 port)
{
	return Open(sockSet,port);
}

bool MgUDPNet::Open(MgNetIPAddress *ip)
{
	return Open(sockSet,ip->port);
}

bool MgUDPNet::Open(MgNetSocketSet *socketSet,MgNetIPAddress *ip)
{
	return Open(socketSet,ip->port);
}

bool MgUDPNet::Open(MgNetSocketSet *socketSet,Uint16 port)
{
	ipAddress.port = port;
	ipAddress.host = 0;
	
	udpSock = MgNet_UDP_Open(port);
	
	if(udpSock == NULL)
	{
		return false;
	}
	
	MgNet_UDP_AddSocket(socketSet, udpSock);
	
	return true;
}
	
MgNetSocketSet *MgUDPNet::GetSocketSet(void)
{
	return sockSet;
}

MgNetIPAddress * MgUDPNet::GetPeerAddress(void)
{
	if(udpSock == NULL)
	{
		return NULL;
	}
	
	return MgNet_UDP_GetPeerAddress(udpSock,-1);
}

MgNetIPAddress * MgUDPNet::GetPeerAddress(int channel)
{
	if(udpSock == NULL)
	{
		return NULL;
	}
	
	return MgNet_UDP_GetPeerAddress(udpSock,channel);
}

bool MgUDPNet::Bind(MgNetIPAddress *ip)
{
	return Bind(-1,ip);
}

bool MgUDPNet::Bind(int channel,MgNetIPAddress *ip)
{
	if(MgNet_UDP_Bind(udpSock,channel,ip) == -1)
	{
		return false;
	}
	
	return true;
}

void MgUDPNet::Unbind(void)
{
	Unbind(-1);
}

void MgUDPNet::Unbind(int channel)
{
	MgNet_UDP_Unbind(udpSock,channel);
}

bool MgUDPNet::AllocPacket(MgNetUDPPacket *udpPacket,int size)
{
	udpPacket = MgNet_AllocPacket(size);
	
	if(udpPacket == NULL)
	{
		return false;
	}
	
	return true;
}

bool MgUDPNet::AllocPacket(MgNetUDPPacket **udpPacket,int howmany,int size)
{
	udpPacket = MgNet_AllocPacketV(howmany,size);
	
	if(udpPacket == NULL)
	{
		return false;
	}
	
	return true;
}

void MgUDPNet::FreePacket(MgNetUDPPacket *udpPacket)
{
	MgNet_FreePacket(udpPacket);
}

void MgUDPNet::FreePacket(MgNetUDPPacket **udpPacket)
{
	MgNet_FreePacketV(udpPacket);
}

int MgUDPNet::ResizePacket(MgNetUDPPacket *udpPacket,int newSize)
{
	return MgNet_ResizePacket(udpPacket,newSize);
}

void MgUDPNet::ResizePacket(MgNetUDPPacket **udpPackets,int howmany,int newSize)
{
	for(int i=0;i<howmany;i++)
	{
		ResizePacket(udpPackets[i],newSize);
	}
}

bool MgUDPNet::Send(void *data, int len)
{
	return Send((MgNetUDPPacket*)data);
}

bool MgUDPNet::Send(MgNetUDPPacket *udpPacket)
{
	return Send(udpPacket);
}

bool MgUDPNet::Send(MgNetUDPPacket *udpPacket,int channel)
{
	if(MgNet_UDP_Send(udpSock,channel,udpPacket) == 1)
	{
		return true;
	}
	return false;
}

bool MgUDPNet::Send(MgNetUDPPacket **udpPackets,int nbPackets)
{

	if(MgNet_UDP_SendV(udpSock,udpPackets,nbPackets) < nbPackets)
	{
		return false;
	}
	return true;
}

bool MgUDPNet::Receive(void *data,int maxlen)
{
	return Receive((MgNetUDPPacket*)data);
}

bool MgUDPNet::Receive(MgNetUDPPacket *udpPacket)
{
	return Receive(udpPacket,(int*)NULL);
}

bool MgUDPNet::Receive(MgNetUDPPacket *udpPacket,int *nbReveivedPackets)
{
	int nbReadPackets;
	
	nbReadPackets = MgNet_UDP_Recv(udpSock,udpPacket);
	
	if(nbReveivedPackets != NULL)
	{
		*nbReveivedPackets = nbReadPackets;
	}
	
	if(nbReadPackets == -1)
	{
		return false;
	}
	
	return true;
}

bool MgUDPNet::Receive(MgNetUDPPacket **udpPackets)
{
	return Receive(udpPackets,(int *)NULL);
}

bool MgUDPNet::Receive(MgNetUDPPacket **udpPackets,int *nbReveivedPackets)
{
	int nbReadPackets;
	
	nbReadPackets = MgNet_UDP_RecvV(udpSock,udpPackets);
	
	if(nbReveivedPackets != NULL)
	{
		*nbReveivedPackets = nbReadPackets;
	}
	
	if(nbReadPackets == -1)
	{
		return false;
	}
	
	return true;
}

int MgUDPNet::HandleClient(int (*callback)(MgNetGenericSocket *,void *), MgNetGenericSocket * sock, void * data)
{
	if(udpSock == NULL)
	{
		return -1;
	}
	
	if(MgNet_SocketReady(udpSock)) 
	{
		if(sock == NULL)
		{
			return callback((MgNetGenericSocket *)udpSock,data);
		}
		else
		{
			return callback(sock,data);
		}
	}
	return -1;	
}

int MgUDPNet::HandleServer(int (*callback)(MgNetGenericSocket *,void *), MgNetGenericSocket * sock, void * data)
{
	if(udpSock == NULL)
	{
		return -1;
	}
	
	if(MgNet_SocketReady(udpSock)) 
	{
		if(sock == NULL)
		{
			return callback((MgNetGenericSocket *)udpSock,data);
		}
		else
		{
			return callback(sock,data);
		}
	}
	return -1;
}

bool MgUDPNet::CheckSocketSet(Uint32 timeout)
{
	if(sockSet == NULL)
	{
		return false;
	}
	
	if(MgNet_CheckSockets(sockSet,timeout) == -1)
	{
		return false;
	}
	return true;
}

void MgUDPNet::Close(void)
{
	this->~MgUDPNet();
}