// MgThread.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include "MgThread.h"
#include "debug_new.h"

/*! Inits its vars. The thread is constructed in Run(). */
MgThread::MgThread(void * data)
{
	fThread = NULL;
	fCanceled = false;
	fReturn = 0;
	fData = data;
}

/*! If the thread is still running, kills it. Then destroys the thread. */
MgThread::~MgThread()
{
	if (!fCanceled)
	{
		Kill();
		Wait();
	}
}

/*!
	Returns right away.
	
	\retval Zero Success.
	\retval Non-Zero Failure.
*/
int 
MgThread::Run()
{
	fThread = SDL_CreateThread(ThreadLauncher, this);
	if (!fThread)
		return -1;
	return 0;
}

void 
MgThread::Cancel()
{
	fCanceled = true;
}

/*!
	Go() should call this once in a while and exit if it returns true;
	
	\retval false Continue running.
	\retval true Please quit.
*/
bool 
MgThread::Canceled() const
{
	return fCanceled;
}

void 
MgThread::Kill()
{
	if (fThread)
	{
		SDL_KillThread(fThread);
		fThread = NULL;
	}
}

int 
MgThread::GetReturnValue()
{
	return fReturn;
}

/*!
	Make sure you have Cancel() before calling this, or you will
	be in a deadlock, as the thread does not no that it needs to quit!
	
	\return The return value of the thread.
*/
int 
MgThread::Wait()
{
	if (fThread)
		SDL_WaitThread(fThread, NULL);	// it's also being waited on in ThreadLauncher()
	return fReturn;
}

// private function used to launch threads.
int 
MgThread::ThreadLauncher(void * thread)
{
	MgThread * t = (MgThread *)thread;
	t->fReturn = t->Go(t->fData);
//	t->fThread = NULL;
	return t->fReturn;
}

