// MgThread.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGTHEAD_H
#define MGTHEAD_H

#include <SDL_thread.h>

#include "MgBeginCode.h"

/*! \file MgThread.h
	Contains the MgThread definition.
*/

/*!
	This class provides a threading interface. To use it, inherit from MgThread and ovverride the protected
	Go() function. When the thread is run, this function is called and should od whatever it needs to do.
*/
class MGDLL MgThread
{
public:
	//! Constructor.
	MgThread(void * data);
	//! Destructor.
	virtual ~MgThread();
	
	//! Runs the thread.
	int Run();
	//! Stops the thread.
	void Cancel();
	//! Checks to see if the thread was stopped.
	bool Canceled() const;
	
	//! Kills the thread.
	void Kill();
	//! Get the return value of the thread.
	int GetReturnValue();
	
	//! Waits for thread to exit.
	int Wait();
	
	//! SDL_Thread * operator.
	operator SDL_Thread *() const { return fThread; }
	//! Gets the SDL_Thread *.
	SDL_Thread * Thread() const { return fThread; }
	
protected:
	//! This function is called when the thead is run. 'data' is whatever you passed to the constructor.
	virtual int Go(void * data) { return 0; }	// default does nothing
	
private:
	SDL_Thread * fThread;
	bool fCanceled;
	int fReturn;
	void * fData;
	
	static int ThreadLauncher(void * thread);
};

#include "MgEndCode.h"

#endif

