// MgTimer.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGTIMER_H
#define MGTIMER_H

/*! \file MgTimer.h
	Contains the MgTimer class.
*/

#include <SDL_timer.h>

#include "MgBeginCode.h"

/*! Inherit from this class and reimplement the Interval() and Callback()
	methods.
*/
class MGDLL MgTimer
{
public:
	//! Constructor.
	MgTimer(void * data, Uint32 interval);
	//! Destructor.
	virtual ~MgTimer();

	//! Returns the new interval for the timer.
	virtual Uint32 Interval() const;
	//! This method is called when the specified timer interval has passed.
	virtual void Callback(void * data);
	
private:
	SDL_TimerID fTimerID;
	void * fData;
	Uint32 fInterval;
	
	friend class MgTimerSystem;
};

#include "MgEndCode.h"

#endif

