// MgTimerSystem.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include "MgTimerSystem.h"
#include "MgTimer.h"
#include "MgApp.h"
#include "debug_new.h"

MgTimerSystem::MgTimerSystem()
{
}


MgTimerSystem::~MgTimerSystem()
{
}

bool 
MgTimerSystem::IsRunning() const
{
	return (SDL_WasInit(MG_TIMER) == MG_TIMER) ? true : false;
}

bool 
MgTimerSystem::Init()
{
	bool ret = IsRunning();
	
	if (!ret)
		ret = SDL_InitSubSystem(MG_TIMER) == 0 ? true : false;
	
	return ret;
}

void 
MgTimerSystem::Quit()
{
	if (IsRunning())
		SDL_QuitSubSystem(MG_TIMER);
}

bool 
MgTimerSystem::AddTimer(MgTimer * timer)
{
	if (timer->fTimerID)	// already running!
		return true;
	
	timer->fTimerID = SDL_AddTimer(timer->fInterval, TimerCallback, timer);
	return (timer->fTimerID == NULL) ? false : true;
}

bool 
MgTimerSystem::RemoveTimer(MgTimer * timer)
{
	return (SDL_RemoveTimer(timer->fTimerID) == SDL_TRUE) ? true : false;
}

Uint32 
MgTimerSystem::TimerCallback(Uint32 interal, void * timer)
{
	MgTimer * t = (MgTimer *)timer;
	t->Callback(t->fData);
	return t->Interval();
}

