// MgColor.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include "MgColor.h"

#include "debug_new.h"

/*! Initializes everything to zero, and alpha to 255. */
MgColor::MgColor()
{
	red = green = blue = 0;
	alpha = 255;
}

/*!
	If this color is passed to a function that doesn't do alpha blending,
	then it will not be used.
*/
MgColor::MgColor(Uint8 r, Uint8 g, Uint8 b, Uint8 a)
{
	red = r;
	green = g;
	blue = b;
	alpha = a;
}

/*! Construcst an MgColor from a SDL_Color. */
MgColor::MgColor(const SDL_Color & color)
{
	red = color.r;
	green = color.g;
	blue = color.b;
	alpha = 255;
}

/*! Constructs an MgColor from a another MgColor. */
MgColor::MgColor(const MgColor & color)
{
	red = color.red;
	green = color.green;
	blue = color.blue;
	alpha = color.alpha;
}

/*! Does nothing. */
MgColor::~MgColor()
{
}

void
MgColor::PrintToStream()
{
	printf("MgColor(%d, %d, %d, %d)\n", red, green, blue, alpha);
}
