// MgFont.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include "MgFont.h"
#include "MgApp.h"
#include "MgPainter.h"

#include "debug_new.h"

#include <SDL.h>

#include <string.h>

/*inline Uint32 getpixel(SDL_Surface * fFont, int x, int y)
{
	Uint8 * bits;
	Uint32 bpp;
	
	bpp = fFont->format->BytesPerPixel;
	bits = (Uint8 *)fFont->pixels;
	
	switch (bpp)
	{
	case 1:
		return(*(bits + y * fFont->pitch + x));
		break;
	case 2:
		return(*((Uint16 *)bits + y * fFont->pitch / 2 + x));
		break;
	case 3:
		{	// format/endian independent
			Uint8 r, g, b;
			r = *((bits) + fFont->format->Rshift / 8);
			g = *((bits) + fFont->format->Gshift / 8);
			b = *((bits) + fFont->format->Bshift / 8);
			return(SDL_MapRGB(fFont->format, r, g, b));
		}
		break;
	case 4:
		return(*((Uint32 *)bits + y * fFont->pitch / 4 + x));
		break;
	}
	return(0);
}
*/
/*!	Constructs the image. Be sure to call InitCheck() before using the
	object.
	
	\param file One of the SFont formatted font files.
*/
MgFont::MgFont(const char * file)
	: MgImage(file)
{
	fCharPos = new int[520];
	InitFont();
}

MgFont::MgFont(MgSurface * surface)
	: MgImage(surface)
{
	fCharPos = new int[520];
	InitFont();
}

/*! Frees up allocated memory. */
MgFont::~MgFont()
{
	delete [] fCharPos;
}

/*! \param dst The surface to draw the text on.
	\param x X position.
	\param y Y position.
	\param text The text to draw.
*/
void 
MgFont::DrawText(MgSurface * dst, int x, int y, const char * text)
{
	int ofs, i = 0;
	SDL_Rect srcrect, dstrect;
	
	while (text[i] != '\0')
	{
		if (text[i] == ' ')
		{
			x += fCharPos[2] - fCharPos[1];
			i++;
		}
		else
		{
			ofs = (text[i] - 33) * 2 + 1;
			srcrect.w = dstrect.w = (fCharPos[ofs + 2] + fCharPos[ofs + 1]) / 2 - (fCharPos[ofs] + fCharPos[ofs - 1])/ 2;
			srcrect.h = dstrect.h = fHeight;
			srcrect.x = (fCharPos[ofs] + fCharPos[ofs - 1]) / 2;
			srcrect.y = 1;
			dstrect.x = (Sint16)(x - (float)(fCharPos[ofs] - fCharPos[ofs - 1]) / 2);
			dstrect.y = y;
			
//			printf("srcrect(%d, %d, %d, %d), dstrect(%d, %d, %d, %d)\n", srcrect.x, srcrect.y, srcrect.w, srcrect.h,
//																		 dstrect.x, dstrect.y, dstrect.w, dstrect.h);
			//SDL_BlitSurface(fSurface, &srcrect, dst->fSurface, &dstrect);
//			printf("blitting\n");
			dst->Blit(this, srcrect, dstrect.x, dstrect.y);
//				printf("blit error\n");
			
			x += fCharPos[ofs + 1] - fCharPos[ofs];
			i++;
		}
	}
}

void 
MgFont::Colorize(MgColor newColor)
{
	int y, x, factor;
	Uint32 color;
	MgColor pixel;
	
	for(y = 1; y < fSurface->h; y++) {
		for(x = 0; x < fSurface->w; x++) {
			/* Get pixel color */
//			temp = GetPixel(Font->Surface, x, y);
			color = GetPixel(x, y);
//			if(temp != Font->Surface->format->colorkey) { /* ignore clear parts */
			if (color != fPixelFormat->ColorKey())
			{
				pixel = fPixelFormat->GetColor(color);
//				printf("not colorkey\n");
/*				SDL_GetRGB(temp, Font->Surface->format, 
					&pixel.r, &pixel.g, &pixel.b);*/

				/* transform pixel */
				factor = (pixel.red + pixel.green + pixel.blue) / 3;
				pixel.red = (newColor.red + factor) >> 1;
				pixel.green = (newColor.green + factor) >> 1;
				pixel.blue = (newColor.blue + factor) >> 1;

				/* write pixel back */
//				temp = SDL_MapRGB(Font->Surface->format, 
//					pixel.r, pixel.g, pixel.b);
//				SetPixel(Font->Surface, x, y, temp);
				PutPixel(x, y, pixel);
			}
		}
	}
}

bool
MgFont::Load(const char * file)
{
	if (!MgImage::Load(file))
		return false;
	InitFont();
	return true;
}

int 
MgFont::Height()
{
	return fHeight;
}

/*! \param text The text to figure the length for.
	\return The length of the text in pixels.
*/
int 
MgFont::Length(const char * text)
{
	int ofs = 0, i = 0, x = 0;
	
	while (text[i] != '\0')
	{
		if (text[i] == ' ')
		{
			x += fCharPos[2] - fCharPos[1];
			i++;
		}
		else
		{
			ofs = (text[i] - 33) * 2 + 1;
			x += fCharPos[ofs + 1] - fCharPos[ofs];
			i++;
		}
		
	}
	return x;
}

/*! This input function is the one from SFont with major modifications by your's truly.
	As you may notice, the text and	textLength parameters are references (text is a
	reference to a pointer). To begin input, you can do one of two things. Initialize
	'text' yourself as a "new char[length]". 'textLength' is then set to that length
	minus one! When this method has input the passed textLength, it will reallocate
	the 'text' to allow for more room as long as it is not longer than 'pixelWidth'
	pixels. When the method returns, you will have a pointer to	that new text pointer
	and 'textLength' will be the maximum length of that array. This way you can keep
	track of everything that has happened and how much room in 'text' is left for
	anything you want to do. The other way to do this would be to initialize a text
	char * to NULL and pass it to this method. It will then allocate a string of
	20 characters long. (BTW, 'textLength' will <b>always<b> be one less the actual amount
	allocated using "new"). You will then have a valid pointer when this method returns.
	
	Example:
	
	char * someText = NULL;
	int length = 0;
	MgFont font = new MgFont("file.ext");
	
	font->Input(surface, x, y, 400, someText, length);
	
	// now we have a valid pointer.
	printf("The input text was %s and the amount of char's allocated is %d\n", someText, length);
	
	Nice, eh?
	
	NOTE:
		Because of this method, if you built libMgTk with DEBUG defined, then it will
		report unfreed memory in this method. This is OK, because the user is supposed
		to free the returned char *.
*/
void 
MgFont::Input(MgSurface * dst, int x, int y, int pixelWidth, char *& text, int & textLength)
{
	SDL_Event event;
	
	int ch = -1, blink = 0;
	Uint32 blinkTimer = 0;
	MgPainter * back;
	SDL_Rect rect;
	SDL_Rect backRect;
	
//	if (!MgApp::App()->PauseFilter())
//		printf("Couldn't lock filters\n");
	
	event.type = 0;
	
//	printf("New MgPainter\n");
	back = new MgPainter(dst->Flags(), dst->Width(), Height(), dst->Format()->BitsPerPixel(),
							dst->Format()->Mask().Rmask, dst->Format()->Mask().Gmask,
							dst->Format()->Mask().Bmask, 0);
	back->FreeOnDelete(true);
	
	rect.x = 0;
	rect.y = y;
	rect.w = dst->Width();
	rect.h = Height();
	
	if (text == NULL)
	{
//		printf("Text is NULL\n");
		text = new char[21];
		textLength = 20;
		strcpy(text, "");
	}
	
//	printf("Blitting to back\n");
	back->Blit(dst, rect, 0, 0);
//	printf("First draw\n");
	DrawText(dst, x, y, text);
	dst->UpdateRects(&rect, 1);
	
	backRect.x = 0;
	backRect.y = 0;
	backRect.w = back->Width();
	backRect.h = back->Height();
	
	SDL_EnableUNICODE(1);
	
	blinkTimer = MgApp::GetTicks();
	
	while (ch != SDLK_RETURN)
	{
		if (event.type == SDL_KEYDOWN)
		{
			ch = event.key.keysym.unicode;
			if (((ch > 31) || (ch == '\b')) && (ch < 128))
			{
				if ((ch == '\b') && (strlen(text) > 0))
				{
					text[strlen(text) - 1] = '\0';
				}
				else if (ch != '\b')
				{
//					printf("length = %d\n", strlen(text));
					if ((int)strlen(text) == textLength) // maxed out
					{
//						printf("strlen == textLength\n");
						char * newText = new char[textLength + 6];
						strcpy(newText, text);
						textLength += 5;
//						printf("delete old text\n");
						delete [] text;
						text = newText;
					}
					sprintf(text, "%s%c", text, ch);
				}
				
				if (Length(text) > pixelWidth)
				{
//					printf("Length > pixelWidth\n");
					text[strlen(text) - 1] = '\0';
				}
				
				dst->Blit(back, backRect, rect);
				DrawText(dst, x, y, text);
				dst->UpdateRects(&rect, 1);
				MgApp::App()->WaitRawEvent(event);
//				SDL_WaitEvent(&event);
			}
		}
		if (MgApp::GetTicks() > blinkTimer)
		{
			blink = 1 - blink;
			blinkTimer = MgApp::GetTicks() + 500;
			if (blink)
			{
				DrawText(dst, x + Length(text), y, "|");
				dst->UpdateRects(&rect, 1);
			}
			else
			{
				dst->Blit(back, backRect, rect);
				DrawText(dst, x, y, text);
				dst->UpdateRects(&rect, 1);
			}
		}
		MgApp::Delay(1);
		MgApp::App()->GetRawEvent(event);
		//SDL_PollEvent(&event);
	}
	dst->Blit(back, backRect, rect);
	dst->UpdateRects(&rect, 1);
	
	text[strlen(text)] = '\0';
	delete back;
	
//	MgApp::App()->ResumeFilter();
}

void 
MgFont::XCentered(MgSurface * dst, int x, int y, const char * text)
{
	DrawText(dst, (x - (Length(text) / 2)), y, text);
}

void 
MgFont::YCentered(MgSurface * dst, int x, int y, const char * text)
{
	DrawText(dst, x, (y - (Height() / 2)), text);
}

void 
MgFont::Centered(MgSurface * dst, int x, int y, const char * text)
{
	DrawText(dst, (x - (Length(text) / 2)), (y - (Height() / 2)), text);
}

void 
MgFont::InitFont()
{
	int x = 0, i = 0;
	Uint32 col;
//	Uint32 col = SDL_MapRGB(fSurface->format, 255, 0, 255);
	
	if (!InitCheck())
		return;	// so we don't crash
	
	col = fPixelFormat->GetPixel(MgColor(255, 0, 255));
	
	fHeight = fSurface->h - 1;
	while (x < fSurface->w)
	{
//		if (getpixel(x, 0) == SDL_MapRGB(fFont->Surface()->format, 255, 0, 255))
		if (GetPixel(x, 0) == col)
		{
//			printf("Got transparent pixel\n");
			fCharPos[i++] = x;
			while ((x < fSurface->w - 1) && (GetPixel(x, 0) == col))
			{
//				printf("still clear\n");
				x++;
			}
			fCharPos[i++] = x;
		}
		x++;
	}
	
	SetColorKey(SDL_SRCCOLORKEY, GetPixel(0, fHeight));
//		printf("Colokey failed\n");
//	SDL_SetColorKey(fFont->Surface(), SDL_SRCCOLORKEY, getpixel(0, fFh));
}
