
// MgGLScreen.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/
#ifndef MGGLSCREEN_H
#define MGGLSCREEN_H

/*! \file MgGLScreen.h

	This file contains the SDL OpenGL interface class.
*/

#include <SDL_video.h>

#include "MgBeginCode.h"

//! Define MgGLAttr. This is the same as SDL_GLattr.
#define MgGLAttr SDL_GLattr

/*! When you create an OpenGL window using MgVideoSystem::SetGLVideMode(),
	you get this class. This is pretty much a useless class. It is here
	to seperate the regular 2D drawing surfaces from the OpenGL surface.
*/
class MgGLScreen
{
public:
	//! Get the address of a GL function (for extension functions).
	static void * GetProcAddress(const char * proc);
	//! Set an OpenGL subsystem attribute before initialization.
	static int SetAttribute(MgGLAttr attr, int value);
	//! Get an OpenGL subsystem attribute.
	static int GetAttribute(MgGLAttr attr, int * value);
	
	//! Swap OpenGL buffers (if double-buffer is supported)
	void SwapBuffers();
	
private:
	SDL_Surface * fSurface;
	
	MgGLScreen();
	~MgGLScreen();
	
	friend class MgVideoSystem;
};

#include "MgEndCode.h"

#endif

