// MgImage.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGIMAGE_H
#define MGIMAGE_H

#include "MgSurface.h"

#include "MgBeginCode.h"
/*! \file MgImage.h
	Contains the image loading class.
*/

/*!
	The MgImage class can load and save MgSurface in various image formats. It can load
	the following formats: BMP, GIF, JPG, PCX, PNG, PPM (ascii version only), TGA and TIFF.
	It can save in the following formats: JPG, PNG, PPM (ascii version only) and BMP.
	To save an MgSurface, you do not have to construct an MgImage, since the saving
	methods are static. NOTE: Always call InitCheck() to make sure the image was loaded.
*/
class MGDLL MgImage : public MgSurface
{
public:
	//! Constructor.
	MgImage();
	//! Constructor.
	MgImage(const char * file);
	//! Constructor.
	MgImage(MgSurface * surface);
	//! Destructor.
	virtual ~MgImage();
	
	//! Load an image from the file.
	virtual bool Load(const char * file);
	
	//! Save as a JPG
	static bool SaveJPG(const char * file, MgSurface * src);
	//! Save as a PNG
	static bool SavePNG(const char * file, MgSurface * src);
	//! Save as a PPM
	static bool SavePPM(const char * file, MgSurface * src);
	//! Save as a BMP
	static bool SaveBMP(const char * file, MgSurface * src);
	
	//! This will be 'true' no matter what. The surface will be freed when this object is deleted.
	virtual void FreeOnDelete(bool free);

	virtual bool InitCheck() const;
	
private:
	bool LoadImage(const char * file);

//	friend class MgSurface;
};

#include "MgEndCode.h"

#endif

