// MgPainter.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include "MgPainter.h"

#include "debug_new.h"

#include <stdio.h>

/*! Does nothing. Do not use this constructor. (That's why it's private...) */
MgPainter::MgPainter()
	: MgSurface()
{
}

/*!
	Creates the surface from one of SDL's surfaces. It also sets FreeOnDelete() to true.
	
	\param surface The source surface.
*/
MgPainter::MgPainter(SDL_Surface * surface)
	: MgSurface(surface)
{
	fFree = true;
}

//@{
/*!
	Creates the surface from the passed vars. It also sets FreeOnDelete() to true.
	
	\param flags/pixels The flags/pixel data for the surface.
	\param width Width.
	\param height Height.
	\param depth Bits per pixel of the surface.
	\param pitch The pitch for the surface.
	\param Rmask/Gmask/Bmask/Amask The red, green, blue and alpha masks for the surface.
*/
MgPainter::MgPainter(Uint32 flags, int width, int height, int depth, Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask)
	: MgSurface(flags, width, height, depth, Rmask, Gmask, Bmask, Amask)
{
	fFree = true;
}


MgPainter::MgPainter(void * pixels, int width, int height, int depth, int pitch, Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask)
	: MgSurface(pixels, width, height, depth, pitch, Rmask, Gmask, Bmask, Amask)
{
	fFree = true;
}
//@}

/*! Does nothing */
MgPainter::~MgPainter()
{
}

void 
MgPainter::FreeOnDelete(bool free)
{
	fFree = free;
}
