// MgPainter.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGPAINTER_H
#define MGPAINTER_H

/*! \file MgPainter.h
	Contains the MgPainter class.
*/

#include "MgSurface.h"
#include "MgVideoSystem.h"
#include "MgBeginCode.h"

/*!
	The surface painter. Use this to draw onto SDL_Surface's. Currently, it
	is identical to MgSurface, but it may change in the future. So be safe
	and use the MgPainter to do your drawing.
	
	\note Currently, if you want to do alpha blending, use a bit depth
		  of 16, 24 or 32. 8 bits-per-pixel currently does not support
		  alpha blending as it is a paletted mode.
*/
class MGDLL MgPainter : public MgSurface
{
public:
	//! Constructor.
	MgPainter(SDL_Surface * surface);
	//! Constructor.
	MgPainter(Uint32 flags, int width, int height, int depth, Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask);
	//! Constructor.
	MgPainter(void * pixels, int width, int height, int depth, int pitch, Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask);
	//! Destructor.
	virtual ~MgPainter();

	virtual void FreeOnDelete(bool free);
	
private:
	//! Constructor.
	MgPainter();
};

#include "MgEndCode.h"

#endif

