// MgPalette.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include "MgColor.h"
#include "MgPalette.h"
#include "MgSurface.h"
#include "debug_new.h"


MgPalette::MgPalette()
{
	fNumColors = 256;
	fColors = new MgColor[256];
	for (int i = 0; i < 256; i++)
		fColors[i](0, 0, 0, 255);
}


MgPalette::MgPalette(SDL_Surface * surface)
{
	SDL_Palette * pal = surface->format->palette;
	fNumColors = pal->ncolors;
	
	fColors = new MgColor[fNumColors];
	for (int i = 0; i < fNumColors; i++)
	{
		fColors[i].red = pal->colors[i].r;
		fColors[i].green = pal->colors[i].g;
		fColors[i].blue = pal->colors[i].b;
		fColors[i].alpha = 255;	// not used in paletted surfaces
	}
}


MgPalette::MgPalette(MgSurface * surface)
{
	SDL_Palette * pal = surface->fSurface->format->palette;

	fNumColors = pal->ncolors;
	
	fColors = new MgColor[fNumColors];
	for (int i = 0; i < fNumColors; i++)
	{
		fColors[i].red = pal->colors[i].r;
		fColors[i].green = pal->colors[i].g;
		fColors[i].blue = pal->colors[i].b;
		fColors[i].alpha = 255;	// not used in paletted surfaces
	}
	
}


MgPalette::~MgPalette()
{
	delete [] fColors;
}

bool 
MgPalette::SetColor(int index, const MgColor & color)
{
	if (index < fNumColors && index >= 0)
	{
		fColors[index] = color;
		return true;
	}
	return false;
}

bool
MgPalette::AddColor(const MgColor & color)
{
	if (fNumColors > 255)	// only allow 256 colors
		return false;
		
	MgColor * newColors = new MgColor[++fNumColors];
	
	for (int i = 0; i < (fNumColors - 1); i++)
		newColors[i] = fColors[i];
	newColors[fNumColors - 1] = color;
	delete [] fColors;
	fColors = newColors;
	return true;
}

MgColor 
MgPalette::GetColor(int index)
{
	if (index >= 0 && index <= 255)
		return fColors[index];
	// otherwise...
	return MgColor(0, 0, 0, 0);	// transparent (alpha == 0)
}

int 
MgPalette::NumColors() const
{
	return fNumColors;
}

void 
MgPalette::SetSize(int size)
{
	if (size < 1 || size > 256)
		return;	// invalid size
	MgColor * newColors = new MgColor[size];
	
	int counter = (size > fNumColors) ? fNumColors : size;
	
	for (int i = 0; i < counter; i++)
		newColors[i] = fColors[i];

	fNumColors = size;
	delete [] fColors;
	fColors = newColors;
}


void 
MgPalette::ApplyTo(MgSurface * surface)
{
	SDL_Color * col = new SDL_Color[fNumColors];
	for (int i = 0; i < fNumColors; i++)
		col[i] = fColors[i];
	SDL_SetColors(surface->fSurface, col, 0, fNumColors);	
}

void 
MgPalette::ApplyTo(SDL_Surface * surface)
{
	SDL_Color * col = new SDL_Color[fNumColors];
	for (int i = 0; i < fNumColors; i++)
		col[i] = fColors[i];
	SDL_SetColors(surface, col, 0, fNumColors);
}

