// MgPixelFormat.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include "MgPalette.h"
#include "MgPixelFormat.h"
#include "debug_new.h"

MgPixelFormat::MgPixelFormat()
{
	fFormat = NULL;
	fPal = NULL;
}

/*! Initializes the pixel format using a SDL_PixelFormat */
MgPixelFormat::MgPixelFormat(SDL_PixelFormat * fmt)
{
	fFormat = fmt;
	fPal = NULL;
	
	if (BytesPerPixel() == 1)
	{
		SDL_Palette * pal = fmt->palette;
		fPal = new MgPalette();
		MgColor col;
		for (int i = 0; i < pal->ncolors; i++)
		{
			col(pal->colors[i].r, pal->colors[i].g, pal->colors[i].b, 255);
			fPal->SetColor(i, col);
		}
	}
}

/*! Destroys the palette if one exists. Does not harm SDL_Surface's palette. */
MgPixelFormat::~MgPixelFormat()
{
	if (fPal)
		delete fPal;
}

MgPalette *
MgPixelFormat::Palette() const
{
	return fPal;
}

Uint32 
MgPixelFormat::GetPixel(MgColor color) const
{
	Uint32 retval;
	if (BytesPerPixel() > 1)	// packed-pixel
	{
		retval = (((color.red >> fFormat->Rloss) << fFormat->Rshift) | ((color.green >> fFormat->Gloss) << fFormat->Gshift) |
					((color.blue >> fFormat->Bloss) << fFormat->Bshift) | ((color.alpha >> fFormat->Aloss) << fFormat->Ashift));
	}
	else		// let SDL handle it (indexed)
	{
		retval = SDL_MapRGB(fFormat, color.red, color.green, color.blue);
	}
	return retval;
}

MgColor 
MgPixelFormat::GetColor(Uint32 pixel) const
{
	MgColor col;
	
	if (BytesPerPixel() > 1)
	{
		col.red = (pixel & fFormat->Rmask) >> fFormat->Rshift << fFormat->Rloss;
		col.green = (pixel & fFormat->Gmask) >> fFormat->Gshift << fFormat->Gloss;
		col.blue = (pixel & fFormat->Bmask) >> fFormat->Bshift << fFormat->Bloss;
		col.alpha = (pixel & fFormat->Amask) >> fFormat->Ashift << fFormat->Aloss;
	}
	else
	{
		SDL_GetRGB(pixel, fFormat, &col.red, &col.green, &col.blue);
	}
	return col;
}

Uint8 
MgPixelFormat::BitsPerPixel() const
{
	return fFormat->BitsPerPixel;
}

Uint8 
MgPixelFormat::BytesPerPixel() const
{
	return fFormat->BytesPerPixel;
}

MgPixelLoss 
MgPixelFormat::Loss() const
{
	return MgPixelLoss(fFormat->Rloss, fFormat->Gloss, fFormat->Bloss, fFormat->Aloss);
}

MgPixelShift 
MgPixelFormat::Shift() const
{
	return MgPixelShift(fFormat->Rshift, fFormat->Gshift, fFormat->Bshift, fFormat->Ashift);
}

MgPixelMask 
MgPixelFormat::Mask() const
{
	return MgPixelMask(fFormat->Rmask, fFormat->Gmask, fFormat->Bmask, fFormat->Amask);
}

bool 
MgPixelFormat::InitCheck() const
{
	return (fFormat == NULL) ? false : true;
}

Uint32
MgPixelFormat::ColorKey() const
{
	return fFormat->colorkey;
}
