// MgRect.cpp
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#include "MgRect.h"
#include "debug_new.h"

/*!
	Creates the rect from the passed vars. They are all optional.
*/
MgRect::MgRect(int _left, int _top, int _right, int _bottom)
{
	left = _left;
	top = _top;
	right = _right;
	bottom = _bottom;
}

/*! Creates the rect from a SDL_Rect */
MgRect::MgRect(const SDL_Rect & rect)
{
	left = rect.x;
	top = rect.y;
	right = rect.x + rect.w - 1;
	bottom = rect.y + rect.h - 1;
}

/*! Creates the rect from another MgRect */
MgRect::MgRect(const MgRect & rect)
{
	left = rect.left;
	top = rect.top;
	right = rect.right;
	bottom = rect.bottom;
}

/*! Does nothing */
MgRect::~MgRect()
{
}

MgPoint 
MgRect::LeftTop() const
{
	return MgPoint(left, top);
}

MgPoint 
MgRect::LeftBottom() const
{
	return MgPoint(left, bottom);
}

MgPoint 
MgRect::RightTop() const
{
	return MgPoint(right, top);
}

MgPoint 
MgRect::RightBottom() const
{
	return MgPoint(right, bottom);
}

int 
MgRect::Width() const
{
	return right - left + 1;
}

int 
MgRect::Height() const
{
	return bottom - top + 1;
}

void 
MgRect::SetTo(int left, int top, int right, int bottom)
{
	this->left = left;
	this->top = top;
	this->right = right;
	this->bottom = bottom;
}

void 
MgRect::InsetBy(int x, int y)
{
	left += x;
	right -= x;
	top += y;
	bottom -= y;
}

